# -*- coding: utf-8 -*-

"""Provides test data for unit tests"""


from .card import Card, CardActions, CardBody, CardHeader, CardMediaImage, CardMediaActions, CardSubtitle, CardTitle, \
    CardMedia
from .components import Button, FAB, Image
from .navbar import NavBar, NavBarLinkItem, NavBarDropDownItem, NavBarDropDownDivider


class DemoTitle1(CardTitle):
    text = "Title goes here"
    size = 2


class DemoHeaderTitle1(CardTitle):
    text = "Two-line item"
    size = 3


class DemoSubtitle1(CardSubtitle):
    text = "Secondary text"


class DemoMediaImage1(CardMediaImage):
    image = Image(source="http://propeller.in/assets/images/profile-pic.png", responsive=True)


class DemoMediaImage2(CardMediaImage):
    image = Image(source="http://propeller.in/assets/images/profile-pic.png", width=80, height=80)


class DemoMediaImage3(CardMediaImage):
    pass


class DemoMedia1(CardMedia):
    content = []


class DemoMedia2(CardMedia):
    content = [DemoMediaImage1(), ]


class DemoMedia3(CardMedia):
    content = [DemoTitle1, DemoSubtitle1, DemoMediaImage2(), ]
    style_inline = True


class DemoMedia4(CardMedia):
    content = None


class DemoMedia5(DemoMedia4):
    style_inline = True


class DemoBody1(CardBody):
    text = "Cards provide context and an entry point to more robust information and views. " \
           "Don't overload cards with extraneous information or actions."


class DemoHeader1(CardHeader):
    content_middle = [DemoHeaderTitle1(), DemoSubtitle1()]
    content_left = [Image("http://propeller.in/assets/images/avatar-icon-40x40.png", width=40, height=40, avatar=True)]


class DemoActions1(CardActions):
    items = [
        Button(content='primary', button_class='btn-primary'),
        Button('Action'),
        Button('third')
    ]


class DemoMediaActions1(CardMediaActions):
    items = [
        FAB('', button_class='btn-primary', icon='share', style='flat', size='sm'),
        FAB('', button_class='btn-primary', icon='thumb_up', style='flat', size='sm'),
        FAB('', button_class='btn-primary', icon='drafts', style='flat', size='sm')
    ]


class DemoCard1(Card):
    primary_title = DemoTitle1()
    secondary_title = DemoSubtitle1()
    header = DemoHeader1()
    actions = DemoActions1()
    media_actions = DemoMediaActions1()
    media = DemoMediaImage1()
    body = DemoBody1()


class DemoCard2(DemoCard1):
    body = None
    header = None
    actions = None


class DemoCard3(DemoCard1):
    style_inverse = True


class DemoCard4(DemoCard1):
    style_inline = True
    body = None
    header = None
    media_actions = None
    media = DemoMediaImage2()


class DemoCard5(DemoCard4):
    media = DemoMedia3()


class TestNavbar1(NavBar):
    brandname = 'propeller-test'
    brandurl = 'https://github.com/tfroehlich82/django-propeller'
    items = [
        NavBarLinkItem('Test1'),
        NavBarDropDownItem('Test2', items=[
            NavBarLinkItem('Test3'),
            NavBarLinkItem('Test4'),
            NavBarDropDownDivider(),
            NavBarLinkItem('Test5'),
        ]),
    ]


class TestNavbar2(TestNavbar1):
    style_inverse = True
    style_static = False
    brandurl = None


class TestNavbar3(TestNavbar1):
    style_inverse = False
    style_static = False


class TestNavbar4(TestNavbar1):
    brandurl = 'test'
    items = [
        NavBarLinkItem('Test1', url='http://example.org'),
        NavBarDropDownItem('Test2', items=[
            NavBarLinkItem('Test3', url='test'),
            NavBarLinkItem('Test4'),
            NavBarDropDownDivider(),
            NavBarLinkItem('Test5'),
        ]),
    ]




# -*- coding: utf-8 -*-
from __future__ import unicode_literals


class PropellerException(Exception):

    """Any exception from this package"""

    pass


class PropellerError(PropellerException):

    """Any exception that is an error"""

    pass



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.utils.safestring import mark_safe

from .exceptions import PropellerError
from .utils import render_tag, add_css_class
from .text import text_value, text_concat

# ToDo: Refactoring


def render_icon(icon, size='sm', **kwargs):
    """Render a Google icon"""
    attrs = {
        'class': add_css_class(
            'material-icons pmd-{size}'.format(size=size),
            kwargs.get('extra_classes', ''),
        )
    }
    title = kwargs.get('title')
    if title:
        attrs['title'] = title
    return render_tag('i', attrs=attrs, content=icon)


def render_bootstrap_icon(icon, **kwargs):
    """Render a Bootstrap glyphicon icon"""
    attrs = {
        'class': add_css_class(
            'glyphicon glyphicon-{icon}'.format(icon=icon),
            kwargs.get('extra_classes', ''),
        )
    }
    title = kwargs.get('title')
    if title:
        attrs['title'] = title
    return render_tag('span', attrs=attrs)


def render_alert(content, alert_type=None, dismissable=True):
    """Render a Bootstrap alert"""
    button = ''
    if not alert_type:
        alert_type = 'info'
    css_classes = ['alert', 'alert-' + text_value(alert_type)]
    if dismissable:
        css_classes.append('alert-dismissable')
        button = '<button type="button" class="close" ' + \
                 'data-dismiss="alert" aria-hidden="true">&times;</button>'
    button_placeholder = '__BUTTON__'
    return mark_safe(render_tag(
        'div',
        attrs={'class': ' '.join(css_classes)},
        content=button_placeholder + text_value(content),
    ).replace(button_placeholder, button))


class Image(object):

    """Render an image object"""

    source = ""
    link = None
    width = None
    height = None
    responsive = False
    avatar = False

    def __init__(self, source="", link=None, width=None, height=None, responsive=False, avatar=False):
        self.source = source
        self.link = link
        self.width = width
        self.height = height
        self.responsive = responsive
        self.avatar = avatar

    def as_html(self):
        img_str = ''
        if self.link:
            img_str += '<a'
            if self.avatar:
                img_str += ' class="avatar-list-img"'
            img_str += '>'
        img_str += '<img src="%s"' % self.source
        if self.width:
            img_str += ' width="%d"' % int(self.width)
        if self.height:
            img_str += ' height="%d"' % int(self.height)
        if self.responsive:
            img_str += ' class="img-responsive"'
        img_str += '>'
        if self.link:
            img_str += '</a>'
        return img_str


class Button(object):

    """Render a button with content"""

    attrs = {}
    content = ""
    classes = []

    def __init__(self, content, button_type='button', icon=None, button_class=None, size=None,
                 href=None, name=None, value=None, title=None, style='default', extra_classes='', _id=''):
        pmd_class = 'pmd-ripple-effect'
        if not button_class:
            button_class = 'btn-default'
        self.classes = add_css_class('btn', button_class)
        self.classes = add_css_class(self.classes, pmd_class)
        size = text_value(size).lower().strip()
        self.content = content
        if size == 'xs':
            self.classes = add_css_class(self.classes, 'btn-xs')
        elif size == 'sm' or size == 'small':
            self.classes = add_css_class(self.classes, 'btn-sm')
        elif size == 'lg' or size == 'large':
            self.classes = add_css_class(self.classes, 'btn-lg')
        elif size == 'md' or size == 'medium':
            pass
        elif size:
            raise PropellerError(
                'Parameter "size" should be "xs", "sm", "lg" or ' +
                'empty ("{}" given).'.format(size))
        if button_type:
            if button_type not in ('submit', 'reset', 'button', 'link'):
                raise PropellerError(
                    'Parameter "button_type" should be "submit", "reset", ' +
                    '"button", "link" or empty  ("{}" given).'.format(button_type))
            self.attrs['type'] = button_type
        if style not in ('default', 'raised', 'flat', 'outline'):
            raise PropellerError(
                'Parameter "style" should be "default", "raised", ' +
                '"flat", "outline" or empty  ("{}" given).'.format(style))
        else:
            self.classes = add_css_class(self.classes, 'pmd-btn-%s' % style)
            self.classes = add_css_class(self.classes, extra_classes)
        self.icon_content = render_icon(icon) if icon else ''
        if href:
            self.attrs['href'] = href
            self.tag = 'a'
        else:
            self.tag = 'button'
        if _id:
            self.attrs['id'] = _id
        if name:
            self.attrs['name'] = name
        if value:
            self.attrs['value'] = value
        if title:
            self.attrs['title'] = title

    def as_html(self):
        self.attrs['class'] = self.classes
        return render_tag(self.tag, attrs=self.attrs,
                          content=mark_safe(text_concat(self.icon_content, self.content, separator=' ')), )


class FAB(object):

    """Render a floating action button"""

    attrs = {}
    content = ""
    classes = []

    def __init__(self, content, button_type='button', icon=None, button_class=None, size=None,
                 href=None, name=None, value=None, title=None, style='default', extra_classes='', _id=''):
        pmd_class = 'pmd-ripple-effect'
        if not button_class:
            button_class = 'btn-default'
        self.classes = add_css_class('', 'btn')
        size = text_value(size).lower().strip()
        self.content = content
        if size == 'xs':
            self.classes = add_css_class(self.classes, 'btn-xs')
        elif size == 'sm' or size == 'small':
            self.classes = add_css_class(self.classes, 'btn-sm')
        elif size == 'lg' or size == 'large':
            self.classes = add_css_class(self.classes, 'btn-lg')
        elif size == 'md' or size == 'medium':
            pass
        elif size:
            raise PropellerError(
                'Parameter "size" should be "xs", "sm", "lg" or ' +
                'empty ("{}" given).'.format(size))
        self.classes = add_css_class(self.classes, 'pmd-btn-fab')
        if button_type:
            if button_type not in ('submit', 'reset', 'button', 'link'):
                raise PropellerError(
                    'Parameter "button_type" should be "submit", "reset", ' +
                    '"button", "link" or empty  ("{}" given).'.format(button_type))
            self.attrs['type'] = button_type
        if style not in ('default', 'raised', 'flat', 'outline'):
            raise PropellerError(
                'Parameter "style" should be "default", "raised", ' +
                '"flat", "outline" or empty  ("{}" given).'.format(style))
        else:
            self.classes = add_css_class(self.classes, 'pmd-btn-%s' % style)
        self.classes = add_css_class(self.classes, pmd_class)
        self.classes = add_css_class(self.classes, extra_classes)
        self.icon_content = render_icon(icon) if icon else ''
        if href:
            self.attrs['href'] = href
            self.tag = 'a'
        else:
            self.tag = 'button'
        if _id:
            self.attrs['id'] = _id
        if name:
            self.attrs['name'] = name
        if value:
            self.attrs['value'] = value
        if title:
            self.attrs['title'] = title
        self.classes = add_css_class(self.classes, button_class)

    def as_html(self):
        self.attrs['class'] = self.classes
        return render_tag(self.tag, attrs=self.attrs,
                          content=mark_safe(text_concat(self.icon_content, self.content, separator=' ')), )



from django2_propeller.navbar import NavBar, NavBarLinkItem, NavBarDropDownItem, NavBarDropDownDivider


class MainNavBar(NavBar):
    brandname = "django2-propeller"
    brandurl = "https://github.com/RaddishIoW/django2-propeller"
    items = [
        NavBarLinkItem("Home", "home"),
        NavBarLinkItem("Typography", "typo"),
        NavBarLinkItem("Cards", "cards"),
        NavBarLinkItem("Buttons", "buttons"),
        NavBarLinkItem("Floating Action Buttons", "fabs"),
        NavBarLinkItem("Navbar", "navbar"),
        NavBarDropDownItem("Forms", [
            NavBarLinkItem("Form", "form_default"),
            NavBarLinkItem("Formset", "formset_default"),
            NavBarLinkItem("Form by field", "form_by_field"),
            NavBarLinkItem("Form horizontal", "form_horizontal"),
            NavBarLinkItem("Form inline", "form_inline"),
            NavBarLinkItem("Form with files", "form_with_files"),
        ]),
        NavBarLinkItem("Pagination", "pagination"),
        NavBarLinkItem("Miscellaneous", "misc"),
    ]


class DemoNavBar1(NavBar):
    brandname = "Brand"
    items = [
        NavBarLinkItem("Link"),
        NavBarLinkItem("Link"),
        NavBarDropDownItem("DropDown", [
            NavBarLinkItem("Action"),
            NavBarLinkItem("Another action"),
            NavBarLinkItem("Something else here"),
            NavBarDropDownDivider(),
            NavBarLinkItem("Separated link"),
            NavBarDropDownDivider(),
            NavBarLinkItem("One more separated link"),
        ])

    ]


class DemoNavBar2(DemoNavBar1):
    style_inverse = False



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import re

from django import forms
from django.contrib.admin.widgets import AdminSplitDateTime
from django.contrib.messages import constants as DEFAULT_MESSAGE_LEVELS
from django.forms.formsets import formset_factory
from django.template import Context, Template
from django.test import TestCase
from django.urls import NoReverseMatch

from .propeller import PROPELLER_SET_REQUIRED_SET_DISABLED
from .exceptions import PropellerError, PropellerException
from .text import text_value, text_concat
from .utils import add_css_class, render_tag
from .test_data import DemoCard1, DemoCard2, DemoCard3, DemoCard4, TestNavbar1, TestNavbar2, TestNavbar3, \
    NavBarLinkItem, NavBarDropDownDivider, NavBarDropDownItem, NavBar, TestNavbar4, DemoMediaImage1, DemoMediaImage3, \
    DemoMedia1, DemoMedia2, DemoMedia3, DemoCard5, DemoMedia4, DemoMedia5
from .test_results import RESULT_CARD1, RESULT_CARD2, RESULT_CARD3, RESULT_CARD4, RESULT_NAVBAR1, RESULT_NAVBAR2, \
    RESULT_NAVBAR3, RESULT_CARD5

try:
    from html.parser import HTMLParser
except ImportError:
    from HTMLParser import HTMLParser


class TemplateTestCase(TestCase):

    @staticmethod
    def render_template(string, context=None):
        context = context or {}
        context = Context(context)
        return Template(string).render(context)

    def render_template_with_propeller(self, string, context=None):
        return self.render_template('{% load propeller %}' + string, context)

    def render_template_with_form(self, text, context=None):
        """
        Create a template ``text`` that first loads bootstrap3.
        """
        if not context:
            context = {}
        if 'form' not in context:
            context['form'] = TestForm()
        return self.render_template_with_propeller(text, context)

    def render_formset(self, formset=None, context=None):
        """
        Create a template that renders a formset
        """
        if not context:
            context = {}
        context['formset'] = formset
        return self.render_template_with_form('{% propeller_formset formset %}', context)

    def render_form(self, form=None, context=None):
        """
        Create a template that renders a form
        """
        if not context:
            context = {}
        if form:
            context['form'] = form
        return self.render_template_with_form('{% propeller_form form %}', context)

    def render_form_field(self, field, context=None):
        """
        Create a template that renders a field
        """
        form_field = 'form.%s' % field
        return self.render_template_with_form('{% propeller_field ' + form_field + ' %}', context)

    def render_field(self, field, context=None):
        """
        Create a template that renders a field
        """
        if not context:
            context = {}
        context['field'] = field
        return self.render_template_with_form('{% propeller_field field %}', context)

    def get_title_from_html(self, html):
        class GetTitleParser(HTMLParser):
            def __init__(self):
                HTMLParser.__init__(self)
                self.title = None

            def handle_starttag(self, tag, attrs):
                for attr, value in attrs:
                    if attr == 'title':
                        self.title = value

        parser = GetTitleParser()
        parser.feed(html)

        return parser.title


TestCase = TemplateTestCase


RADIO_CHOICES = (
    ('1', 'Radio 1'),
    ('2', 'Radio 2'),
)

MEDIA_CHOICES = (
    ('Audio', (
        ('vinyl', 'Vinyl'),
        ('cd', 'CD'),
    )
     ),
    ('Video', (
        ('vhs', 'VHS Tape'),
        ('dvd', 'DVD'),
    )
     ),
    ('unknown', 'Unknown'),
)


class TestForm(forms.Form):
    """
    Form with a variety of widgets to test bootstrap3 rendering.
    """
    date = forms.DateField(required=False)
    datetime = forms.SplitDateTimeField(widget=AdminSplitDateTime(), required=False)
    subject = forms.CharField(
        max_length=100,
        help_text='my_help_text',
        required=True,
        widget=forms.TextInput(attrs={'placeholder': 'placeholdertest'}),
    )
    password = forms.CharField(widget=forms.PasswordInput)
    message = forms.CharField(required=False, help_text='<i>my_help_text</i>')
    sender = forms.EmailField(
        label='Sender © unicode',
        help_text='E.g., "me@example.com"')
    secret = forms.CharField(initial=42, widget=forms.HiddenInput)
    cc_myself = forms.BooleanField(
        required=False,
        help_text='cc stands for "carbon copy." You will get a copy in your mailbox.'
    )
    select1 = forms.ChoiceField(choices=RADIO_CHOICES)
    select2 = forms.MultipleChoiceField(
        choices=RADIO_CHOICES,
        help_text='Check as many as you like.',
    )
    select3 = forms.ChoiceField(choices=MEDIA_CHOICES)
    select4 = forms.MultipleChoiceField(
        choices=MEDIA_CHOICES,
        help_text='Check as many as you like.',
    )
    category1 = forms.ChoiceField(
        choices=RADIO_CHOICES, widget=forms.RadioSelect)
    category2 = forms.MultipleChoiceField(
        choices=RADIO_CHOICES,
        widget=forms.CheckboxSelectMultiple,
        help_text='Check as many as you like.',
    )
    category3 = forms.ChoiceField(
        widget=forms.RadioSelect, choices=MEDIA_CHOICES)
    category4 = forms.MultipleChoiceField(
        choices=MEDIA_CHOICES,
        widget=forms.CheckboxSelectMultiple,
        help_text='Check as many as you like.',
    )
    addon = forms.CharField(
        widget=forms.TextInput(attrs={'addon_before': 'before', 'addon_after': 'after'}),
    )

    required_css_class = 'bootstrap3-req'

    # Set this to allow tests to work properly in Django 1.10+
    # More information, see issue #337
    use_required_attribute = False

    def clean(self):
        cleaned_data = super(TestForm, self).clean()
        raise forms.ValidationError(
            "This error was added to show the non field errors styling.")
        # return cleaned_data


class TestFormWithoutRequiredClass(TestForm):
    required_css_class = ''


class SettingsTest(TestCase):
    def test_settings(self):
        from .propeller import PROPELLER
        self.assertTrue(PROPELLER)

    def test_propeller_javascript_tag(self):
        res = self.render_template_with_form('{% propeller_javascript %}')
        self.assertEqual(
            res.strip(),
            '<script src="//maxcdn.bootstrapcdn.com/bootstrap/3.3.7/js/bootstrap.min.js"></script>'
        )

    def test_propeller_css_tag(self):
        res = self.render_template_with_form('{% propeller_css %}').strip()
        self.assertIn(
            '<link href="//maxcdn.bootstrapcdn.com/bootstrap/3.3.7/css/bootstrap.min.css" rel="stylesheet">',
            res
        )

    # def test_settings_filter(self):
    #     res = self.render_template_with_form('{{ "required_css_class"|propeller_setting }}')
    #     self.assertEqual(res.strip(), 'bootstrap3-req')
    #     res = self.render_template_with_form('{% if "javascript_in_head"|propeller_setting %}head{% else %}body{% endif %}')
    #     self.assertEqual(res.strip(), 'head')

    def test_required_class(self):
        form = TestForm()
        res = self.render_template_with_form('{% propeller_form form %}', {'form': form})
        self.assertIn('bootstrap3-req', res)

    # def test_error_class(self):
    #     form = TestForm({})
    #     res = self.render_template_with_form('{% propeller_form form %}', {'form': form})
    #     self.assertIn('bootstrap3-err', res)

    # def test_bound_class(self):
    #     form = TestForm({'sender': 'sender'})
    #     res = self.render_template_with_form('{% propeller_form form %}', {'form': form})
    #     self.assertIn('bootstrap3-bound', res)


class TemplateTest(TestCase):
    def test_empty_template(self):
        res = self.render_template_with_form('')
        self.assertEqual(res.strip(), '')

    def test_text_template(self):
        res = self.render_template_with_form('some text')
        self.assertEqual(res.strip(), 'some text')

    def test_propeller_template(self):
        res = self.render_template(
            '{% extends "propeller/propeller.html" %}' +
            '{% block propeller_content %}' +
            'test_propeller_content' +
            '{% endblock %}'
        )
        self.assertIn('test_propeller_content', res)

    def test_javascript_without_jquery(self):
        res = self.render_template_with_form('{% propeller_javascript jquery=0 %}')
        self.assertIn('bootstrap', res)
        self.assertNotIn('jquery', res)

    def test_javascript_with_jquery(self):
        res = self.render_template_with_form('{% propeller_javascript jquery=1 %}')
        self.assertIn('bootstrap', res)
        self.assertIn('jquery', res)


class FormSetTest(TestCase):
    def test_illegal_formset(self):
        with self.assertRaises(PropellerError):
            self.render_formset(formset='illegal')


class FormTest(TestCase):
    def test_illegal_form(self):
        with self.assertRaises(PropellerError):
            self.render_form(form='illegal')

    def test_field_names(self):
        form = TestForm()
        res = self.render_form(form)
        for field in form:
            # datetime has a multiwidget field widget
            if field.name == "datetime":
                self.assertIn('name="datetime_0"', res)
                self.assertIn('name="datetime_1"', res)
            else:
                self.assertIn('name="%s"' % field.name, res)

    def test_field_addons(self):
        form = TestForm()
        res = self.render_form(form)
        self.assertIn('<div class="input-group"><span class="input-group-addon">before</span><input', res)
        self.assertIn('<span class="input-group-addon">after</span></div>', res)

    def test_exclude(self):
        form = TestForm()
        res = self.render_template_with_form(
            '{% propeller_form form exclude="cc_myself" %}', {'form': form})
        self.assertNotIn('cc_myself', res)

    def test_layout_horizontal(self):
        form = TestForm()
        res = self.render_template_with_form(
            '{% propeller_form form layout="horizontal" %}', {'form': form})
        self.assertIn('col-md-3', res)
        self.assertIn('col-md-9', res)
        res = self.render_template_with_form(
            '{% propeller_form form layout="horizontal" ' +
            'horizontal_label_class="hlabel" ' +
            'horizontal_field_class="hfield" %}',
            {'form': form}
        )
        self.assertIn('hlabel', res)
        self.assertIn('hfield', res)

    def test_required_class(self):
        form = TestForm({'sender': 'sender'})
        res = self.render_template_with_form('{% propeller_form form %}', {'form': form})
        self.assertIn('bootstrap3-req', res)

        res = self.render_template_with_form(
            '{% propeller_form form required_css_class="successful-test" %}',
            {'form': form}
        )
        self.assertIn('successful-test', res)

        res = self.render_template_with_form('{% propeller_form form required_css_class="" %}',
                                        {'form': form})
        self.assertNotIn('bootstrap3-req', res)


class FieldTest(TestCase):
    def test_illegal_field(self):
        with self.assertRaises(PropellerError):
            self.render_field(field='illegal')

    def test_show_help(self):
        res = self.render_form_field('subject')
        self.assertIn('my_help_text', res)
        self.assertNotIn('<i>my_help_text</i>', res)
        res = self.render_template_with_form('{% propeller_field form.subject show_help=0 %}')
        self.assertNotIn('my_help_text', res)

    def test_help_with_quotes(self):
        # Checkboxes get special handling, so test a checkbox and something else
        res = self.render_form_field('sender')
        self.assertEqual(self.get_title_from_html(res), TestForm.base_fields['sender'].help_text)
        res = self.render_form_field('cc_myself')
        self.assertEqual(self.get_title_from_html(res), TestForm.base_fields['cc_myself'].help_text)

    def test_subject(self):
        res = self.render_form_field('subject')
        self.assertIn('type="text"', res)
        self.assertIn('placeholder="placeholdertest"', res)

    def test_password(self):
        res = self.render_form_field('password')
        self.assertIn('type="password"', res)
        self.assertIn('placeholder="Password"', res)

    def test_required_field(self):
        if PROPELLER_SET_REQUIRED_SET_DISABLED:
            required_field = self.render_form_field('subject')
            self.assertIn('required', required_field)
            self.assertIn('bootstrap3-req', required_field)
            not_required_field = self.render_form_field('message')
            self.assertNotIn('required', not_required_field)
            # Required field with required=0
            form_field = 'form.subject'
            rendered = self.render_template_with_form('{% propeller_field ' + form_field + ' set_required=0 %}')
            self.assertNotIn('required', rendered)
        else:
            required_css_class = 'bootstrap3-req'
            required_field = self.render_form_field('subject')
            self.assertIn(required_css_class, required_field)
            not_required_field = self.render_form_field('message')
            self.assertNotIn(required_css_class, not_required_field)
        # Required settings in field
        form_field = 'form.subject'
        rendered = self.render_template_with_form(
            '{% propeller_field ' + form_field + ' required_css_class="test-required" %}'
        )
        self.assertIn('test-required', rendered)

    def test_empty_permitted(self):
        """
        If a form has empty_permitted, no fields should get the CSS class for required.
        Django <= 1.8, also check `required` attribute.
        """
        if PROPELLER_SET_REQUIRED_SET_DISABLED:
            required_css_class = 'bootstrap3-req'
            form = TestForm()
            res = self.render_form_field('subject', {'form': form})
            self.assertIn(required_css_class, res)
            form.empty_permitted = True
            res = self.render_form_field('subject', {'form': form})
            self.assertNotIn(required_css_class, res)
        else:
            required_css_class = 'bootstrap3-req'
            form = TestForm()
            res = self.render_form_field('subject', {'form': form})
            self.assertIn(required_css_class, res)
            form.empty_permitted = True
            res = self.render_form_field('subject', {'form': form})
            self.assertNotIn(required_css_class, res)

    def test_input_group(self):
        res = self.render_template_with_form('{% propeller_field form.subject addon_before="$"  addon_after=".00" %}')
        self.assertIn('class="input-group"', res)
        self.assertIn('class="input-group-addon">$', res)
        self.assertIn('class="input-group-addon">.00', res)

    def test_input_group_addon_button(self):
        res = self.render_template_with_form(
            '{% propeller_field form.subject addon_before="$" addon_before_class="input-group-btn" '
            'addon_after=".00" addon_after_class="input-group-btn" %}'
        )
        self.assertIn('class="input-group"', res)
        self.assertIn('class="input-group-btn">$', res)
        self.assertIn('class="input-group-btn">.00', res)

    def test_size(self):
        def _test_size(param, klass):
            res = self.render_template_with_form('{% propeller_field form.subject size="' + param + '" %}')
            self.assertIn(klass, res)

        def _test_size_medium(param):
            res = self.render_template_with_form('{% propeller_field form.subject size="' + param + '" %}')
            self.assertNotIn('input-lg', res)
            self.assertNotIn('input-sm', res)
            self.assertNotIn('input-md', res)

        _test_size('sm', 'input-sm')
        _test_size('small', 'input-sm')
        _test_size('lg', 'input-lg')
        _test_size('large', 'input-lg')
        _test_size_medium('md')
        _test_size_medium('medium')
        _test_size_medium('')

    def test_datetime(self):
        field = self.render_form_field('datetime')
        self.assertIn('vDateField', field)
        self.assertIn('vTimeField', field)

    def test_field_same_render(self):
        context = dict(form=TestForm())
        rendered_a = self.render_form_field("addon", context)
        rendered_b = self.render_form_field("addon", context)
        self.assertEqual(rendered_a, rendered_b)

    def test_label(self):
        res = self.render_template_with_form('{% propeller_label "foobar" label_for="subject" %}')
        self.assertEqual('<label for="subject">foobar</label>', res)

    def test_attributes_consistency(self):
        form = TestForm()
        attrs = form.fields['addon'].widget.attrs.copy()
        context = dict(form=form)
        field_alone = self.render_form_field("addon", context)
        self.assertEqual(attrs, form.fields['addon'].widget.attrs)


class ComponentsTest(TestCase):
    def test_icon(self):
        res = self.render_template_with_form('{% propeller_icon "star" %}')
        self.assertEqual(
            res.strip(), '<i class="material-icons pmd-sm">star</i>')
        res = self.render_template_with_form('{% propeller_icon "star" title="alpha centauri" %}')
        self.assertIn(res.strip(), [
            '<i class="material-icons pmd-sm" title="alpha centauri">star</i>',
            '<span title="alpha centauri" class="glyphicon glyphicon-star"></span>',
        ])

    def test_alert(self):
        res = self.render_template_with_form('{% propeller_alert "content" alert_type="danger" %}')
        self.assertEqual(
            res.strip(),
            '<div class="alert alert-danger alert-dismissable">' +
            '<button type="button" class="close" data-dismiss="alert" ' +
            'aria-hidden="true">' +
            '&times;</button>content</div>'
        )


class MessagesTest(TestCase):
    def test_messages(self):
        class FakeMessage(object):
            """
            Follows the `django.contrib.messages.storage.base.Message` API.
            """
            level = None
            message = None
            extra_tags = None

            def __init__(self, level, message, extra_tags=None):
                self.level = level
                self.extra_tags = extra_tags
                self.message = message

            def __str__(self):
                return self.message

        pattern = re.compile(r'\s+')
        messages = [FakeMessage(DEFAULT_MESSAGE_LEVELS.WARNING, "hello")]
        res = self.render_template_with_form(
            '{% propeller_messages messages %}', {'messages': messages})
        expected = """
    <div class="alert alert-warning alert-dismissable">
        <button type="button" class="close" data-dismiss="alert"
            aria-hidden="true">&#215;</button>
        hello
    </div>
"""
        self.assertEqual(
            re.sub(pattern, '', res),
            re.sub(pattern, '', expected)
        )

        messages = [FakeMessage(DEFAULT_MESSAGE_LEVELS.ERROR, "hello")]
        res = self.render_template_with_form(
            '{% propeller_messages messages %}', {'messages': messages})
        expected = """
    <div class="alert alert-danger alert-dismissable">
        <button type="button" class="close" data-dismiss="alert"
            aria-hidden="true">&#215;</button>
        hello
    </div>
        """
        self.assertEqual(
            re.sub(pattern, '', res),
            re.sub(pattern, '', expected)
        )

        messages = [FakeMessage(None, "hello")]
        res = self.render_template_with_form(
            '{% propeller_messages messages %}', {'messages': messages})
        expected = """
    <div class="alert alert-danger alert-dismissable">
        <button type="button" class="close" data-dismiss="alert"
            aria-hidden="true">&#215;</button>
        hello
    </div>
        """

        self.assertEqual(
            re.sub(pattern, '', res),
            re.sub(pattern, '', expected)
        )

        messages = [FakeMessage(DEFAULT_MESSAGE_LEVELS.ERROR, "hello http://example.com")]
        res = self.render_template_with_form(
            '{% propeller_messages messages %}', {'messages': messages})
        expected = """
    <div class="alert alert-danger alert-dismissable">
        <button type="button" class="close" data-dismiss="alert" aria-hidden="true">&#215;</button>
        hello http://example.com
    </div>        """
        self.assertEqual(
            re.sub(pattern, '', res).replace('rel="nofollow"', ''),
            re.sub(pattern, '', expected).replace('rel="nofollow"', '')
        )

        messages = [FakeMessage(DEFAULT_MESSAGE_LEVELS.ERROR, "hello\nthere")]
        res = self.render_template_with_form(
            '{% propeller_messages messages %}', {'messages': messages})
        expected = """
    <div class="alert alert-danger alert-dismissable">
        <button type="button" class="close" data-dismiss="alert"
            aria-hidden="true">&#215;</button>
        hello there
    </div>
        """
        self.assertEqual(
            re.sub(pattern, '', res),
            re.sub(pattern, '', expected)
        )


class UtilsTest(TestCase):
    def test_add_css_class(self):
        css_classes = "one two"
        css_class = "three four"
        classes = add_css_class(css_classes, css_class)
        self.assertEqual(classes, "one two three four")

        classes = add_css_class(css_classes, css_class, prepend=True)
        self.assertEqual(classes, "three four one two")

    def test_text_value(self):
        self.assertEqual(text_value(''), "")
        self.assertEqual(text_value(' '), " ")
        self.assertEqual(text_value(None), "")
        self.assertEqual(text_value(1), "1")

    def test_text_concat(self):
        self.assertEqual(text_concat(1, 2), "12")
        self.assertEqual(text_concat(1, 2, separator='='), "1=2")
        self.assertEqual(text_concat(None, 2, separator='='), "2")

    def test_render_tag(self):
        self.assertEqual(render_tag('span'), '<span></span>')
        self.assertEqual(render_tag('span', content='foo'), '<span>foo</span>')
        self.assertEqual(
            render_tag('span', attrs={'bar': 123}, content='foo'),
            '<span bar="123">foo</span>'
        )


class ButtonTest(TestCase):
    def test_button(self):
        res = self.render_template_with_form("{% propeller_button 'button' size='lg' %}")
        self.assertEqual('<button class="btn btn-default pmd-ripple-effect btn-lg pmd-btn-default" type="button">'
                         'button</button>', res.strip())
        res = self.render_template_with_form("{% propeller_button 'button' size='lg' href='#' %}")
        self.assertIn('<a class="btn btn-default pmd-ripple-effect btn-lg pmd-btn-default" href="#" type="button">'
                      'button</a>', res.strip())


class ShowLabelTest(TestCase):
    def test_show_label(self):
        form = TestForm()
        res = self.render_template_with_form(
            '{% propeller_form form show_label=False %}',
            {'form': form}
        )
        self.assertIn('sr-only', res)

    def test_for_formset(self):
        TestFormSet = formset_factory(TestForm, extra=1)
        test_formset = TestFormSet()
        res = self.render_template_with_form(
            '{% propeller_formset formset show_label=False %}',
            {'formset': test_formset}
        )
        self.assertIn('sr-only', res)

    def test_button_with_icon(self):
        res = self.render_template_with_form(
            "{% propeller_button 'test' icon='info-sign' %}"
        )
        self.assertEqual('<button class="btn btn-default pmd-ripple-effect pmd-btn-default" href="#" type="button">'
                         '<i class="material-icons pmd-sm">info-sign</i> test</button>', res.strip())
        res = self.render_template_with_form(
            "{% propeller_button 'test' icon='info-sign' button_class='btn-primary' %}"
        )
        self.assertEqual('<button class="btn btn-primary pmd-ripple-effect pmd-btn-default" href="#" type="button">'
                         '<i class="material-icons pmd-sm">info-sign</i> test</button>', res.strip())
        res = self.render_template_with_form(
            "{% propeller_button 'test' icon='info-sign' button_type='submit' %}"
        )
        self.assertEqual('<button class="btn btn-default pmd-ripple-effect pmd-btn-default" href="#" type="submit">'
                         '<i class="material-icons pmd-sm">info-sign</i> test</button>', res.strip())


class TypographyTest(TestCase):
    def test_marked_text_filter(self):
        res = self.render_template_with_propeller(
            '{{ text|pmd_mark_text }}',
            {'text': 'This is a test'}
        )
        self.assertInHTML('<mark>This is a test</mark>', res)

    def test_striked_text_filter(self):
        res = self.render_template_with_propeller(
            '{{ text|pmd_strike_text }}',
            {'text': 'This is a test'}
        )
        self.assertInHTML('<s>This is a test</s>', res)

    def test_underlined_text_filter(self):
        res = self.render_template_with_propeller(
            '{{ text|pmd_underline_text }}',
            {'text': 'This is a test'}
        )
        self.assertInHTML('<u>This is a test</u>', res)

    def test_bold_text_filter(self):
        res = self.render_template_with_propeller(
            '{{ text|pmd_bold_text }}',
            {'text': 'This is a test'}
        )
        self.assertInHTML('<strong>This is a test</strong>', res)

    def test_italic_text_filter(self):
        res = self.render_template_with_propeller(
            '{{ text|pmd_italic_text }}',
            {'text': 'This is a test'}
        )
        self.assertInHTML('<em>This is a test</em>', res)

    def test_lead_text_filter(self):
        res = self.render_template_with_propeller(
            '{{ text|pmd_lead_text }}',
            {'text': 'This is a test'}
        )
        self.assertInHTML('<span class="lead">This is a test</span>', res)

    def test_display_text_filter(self):
        res = self.render_template_with_propeller(
            '{{ text|pmd_display_text:1 }}',
            {'text': 'This is a test'}
        )
        self.assertInHTML('<span class="pmd-display1">This is a test</span>', res)
        res = self.render_template_with_propeller(
            '{{ text|pmd_display_text:2 }}',
            {'text': 'This is a test'}
        )
        self.assertInHTML('<span class="pmd-display2">This is a test</span>', res)
        res = self.render_template_with_propeller(
            '{{ text|pmd_display_text:3 }}',
            {'text': 'This is a test'}
        )
        self.assertInHTML('<span class="pmd-display3">This is a test</span>', res)
        res = self.render_template_with_propeller(
            '{{ text|pmd_display_text:4 }}',
            {'text': 'This is a test'}
        )
        self.assertInHTML('<span class="pmd-display4">This is a test</span>', res)

    def test_muted_text_filter(self):
        res = self.render_template_with_propeller(
            '{{ text|pmd_muted_text }}',
            {'text': 'This is a test'}
        )
        self.assertInHTML('<span class="text-muted">This is a test</span>', res)


class FABsTests(TestCase):
    def test_default_fab(self):
        res = self.render_template_with_propeller(
            '{% propeller_fab "" icon="star" %}',
        )
        self.assertInHTML('<button class="btn pmd-btn-fab pmd-btn-default pmd-ripple-effect btn-default" type="button">'
                          '<i class="material-icons pmd-sm">star</i></button>', res)

    def test_default_fab_with_btn_class(self):
        res = self.render_template_with_propeller(
            '{% propeller_fab "" icon="warning" button_class="btn-warning" %}',
        )
        self.assertInHTML('<button class="btn btn-warning pmd-btn-fab pmd-ripple-effect pmd-btn-default" type="button">'
                          '<i class="material-icons pmd-sm">warning</i></button>', res)

    def test_default_fab_with_link(self):
        res = self.render_template_with_propeller(
            '{% propeller_fab "Link" button_class="btn-link" %}',
        )
        self.assertInHTML('<button class="btn pmd-btn-fab pmd-btn-default pmd-ripple-effect btn-link" type="button">'
                          'Link</button>', res)

    def test_raised_fab(self):
        res = self.render_template_with_propeller(
            '{% propeller_fab "" icon="star" style="raised" %}',
        )
        self.assertInHTML('<button class="btn btn-default pmd-btn-fab pmd-ripple-effect pmd-btn-raised" type="button">'
                          '<i class="material-icons pmd-sm">star</i></button>', res)

    def test_raised_fab_with_btn_class(self):
        res = self.render_template_with_propeller(
            '{% propeller_fab "" icon="error" style="raised" button_class="btn-danger" %}',
        )
        self.assertInHTML('<button class="btn btn-danger pmd-btn-fab pmd-ripple-effect pmd-btn-raised" type="button">'
                          '<i class="material-icons pmd-sm">error</i></button>', res)

    def test_raised_fab_with_link(self):
        res = self.render_template_with_propeller(
            '{% propeller_fab "Link" style="raised" button_class="btn-link" %}',
        )
        self.assertInHTML('<button class="btn btn-link pmd-btn-fab pmd-ripple-effect pmd-btn-raised" type="button">'
                          'Link</button>', res)

    def test_flat_fab(self):
        res = self.render_template_with_propeller(
            '{% propeller_fab "" icon="star" style="flat" %}',
        )
        self.assertInHTML('<button class="btn btn-default pmd-btn-fab pmd-ripple-effect pmd-btn-flat" type="button">'
                          '<i class="material-icons pmd-sm">star</i></button>', res)

    def test_flat_fab_with_btn_class(self):
        res = self.render_template_with_propeller(
            '{% propeller_fab "" icon="info" style="flat" button_class="btn-danger" %}',
        )
        self.assertInHTML('<button class="btn btn-danger pmd-btn-fab pmd-ripple-effect pmd-btn-flat" type="button">'
                          '<i class="material-icons pmd-sm">info</i></button>', res)

    def test_flat_fab_with_link(self):
        res = self.render_template_with_propeller(
            '{% propeller_fab "Link" style="flat" button_class="btn-link" %}',
        )
        self.assertInHTML('<button class="btn btn-link pmd-btn-fab pmd-ripple-effect pmd-btn-flat" type="button">'
                          'Link</button>', res)

    def test_outline_fab(self):
        res = self.render_template_with_propeller(
            '{% propeller_fab "" icon="star" style="outline" %}',
        )
        self.assertInHTML('<button class="btn btn-default pmd-btn-fab pmd-ripple-effect pmd-btn-outline" type="button">'
                          '<i class="material-icons pmd-sm">star</i></button>', res)

    def test_outline_fab_with_btn_class(self):
        res = self.render_template_with_propeller(
            '{% propeller_fab "" icon="check" style="outline" button_class="btn-danger" %}',
        )
        self.assertInHTML('<button class="btn btn-danger pmd-btn-fab pmd-ripple-effect pmd-btn-outline" type="button">'
                          '<i class="material-icons pmd-sm">check</i></button>', res)

    def test_outline_fab_with_link(self):
        res = self.render_template_with_propeller(
            '{% propeller_fab "Link" style="outline" button_class="btn-link" %}',
        )
        self.assertInHTML('<button class="btn btn-link pmd-btn-fab pmd-ripple-effect pmd-btn-outline" type="button">'
                          'Link</button>', res)


class DjangoAppTests(TestCase):
    def test_app_config(self):
        from django2_propeller.apps import DjangoPropellerConfig
        self.assertEqual(DjangoPropellerConfig.name, 'django2_propeller')


class PropellerMixinTests(TestCase):
    class TestNavbar(NavBar):
        pass

    def test_navbar_mixin(self):
        from django2_propeller.views import NavBarMixin, ContextMixin
        test_mixin = NavBarMixin()
        self.assertIsInstance(test_mixin, ContextMixin)
        test_mixin.navbar_class = self.TestNavbar
        self.assertIsInstance(test_mixin.get_context_data().get('navbar')(), self.TestNavbar)


class PropellerNavBarTests(TestCase):
    def test_navbar_config(self):
        self.assertEqual(TestNavbar1().get_brand_url(), 'https://github.com/tfroehlich82/django-propeller')
        self.assertEqual(TestNavbar1().brandname, 'propeller-test')
        self.assertEqual(TestNavbar2().get_brand_url(), 'javascript:void(0);')
        with self.assertRaises(NoReverseMatch):
            TestNavbar4().get_brand_url()

    def test_rendered_template(self):
        res = self.render_template_with_propeller(
            '{% propeller_navbar testnav1 %}',
            context={'testnav1': TestNavbar1()}
        )
        self.assertInHTML(RESULT_NAVBAR1, res)
        res = self.render_template_with_propeller(
            '{% propeller_navbar testnav2 %}',
            context={'testnav2': TestNavbar2()}
        )
        self.assertInHTML(RESULT_NAVBAR2, res)
        res = self.render_template_with_propeller(
            '{% propeller_navbar testnav3 %}', {'testnav3': TestNavbar3()}
        )
        self.assertInHTML(RESULT_NAVBAR3, res)

    def test_navbar_items(self):
        for itm in TestNavbar4().items:
            if itm.name in ('Test1', ):
                self.assertIsInstance(itm, NavBarLinkItem)
                self.assertEqual(itm.get_url(), 'http://example.org')
            elif itm.name in ('Test2', ):
                self.assertIsInstance(itm, NavBarDropDownItem)
                for dd_itm in itm.items:
                    if hasattr(dd_itm, 'name') and dd_itm.name in ('Test3', 'Test4', 'Test5'):
                        if dd_itm.name == 'Test3':
                            with self.assertRaises(NoReverseMatch):
                                dd_itm.get_url()
                        elif dd_itm.name == 'Test4':
                            self.assertEqual(dd_itm.get_url(), 'javascript:void(0);')
                        self.assertIsInstance(dd_itm, NavBarLinkItem)
                    else:
                        self.assertIsInstance(dd_itm, NavBarDropDownDivider)


class PropellerCardTests(TestCase):
    def test_card_media(self):
        res = DemoMediaImage1().as_html()
        self.assertEqual('<img src="http://propeller.in/assets/images/profile-pic.png" class="img-responsive">', res)
        res = DemoMediaImage3().as_html()
        self.assertIsNone(res)
        res = DemoMedia1().as_html()
        self.assertEqual('<div class="pmd-card-media"><div class="media-body"></div></div>', res)
        res = DemoMedia2().as_html()
        self.assertEqual('<div class="pmd-card-media"><div class="media-body">'
                         '<img src="http://propeller.in/assets/images/profile-pic.png" class="img-responsive">'
                         '</div></div>', res)
        res = DemoMedia3().as_html()
        self.assertEqual('<div class="pmd-card-media"><div class="media-body"></div>'
                         '<div class="media-right media-middle">'
                         '<img src="http://propeller.in/assets/images/profile-pic.png" width="80" height="80">'
                         '</div></div>', res)

    def test_rendered_template(self):
        res = self.render_template_with_propeller(
            '{% propeller_card card1 %}',
            context={'card1': DemoCard1()}
        )
        self.assertInHTML(RESULT_CARD1, res)
        res = self.render_template_with_propeller(
            '{% propeller_card card2 %}',
            context={'card2': DemoCard2()}
        )
        self.assertInHTML(RESULT_CARD2, res)
        res = self.render_template_with_propeller(
            '{% propeller_card card3 %}', {'card3': DemoCard3()}
        )
        self.assertInHTML(RESULT_CARD3, res)
        res = self.render_template_with_propeller(
            '{% propeller_card card4 %}', {'card4': DemoCard4()}
        )
        self.assertInHTML(RESULT_CARD4, res)
        res = self.render_template_with_propeller(
            '{% propeller_card card5 %}', {'card5': DemoCard5()}
        )
        self.assertInHTML(RESULT_CARD5, res)

    def test_exceptions(self):
        with self.assertRaises(PropellerException):
            DemoMedia4().get_media_body()
        with self.assertRaises(PropellerException):
            DemoMedia5().get_media_body_inline()



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.contrib import messages
from django.core.files.storage import default_storage
from django.core.paginator import Paginator, PageNotAnInteger, EmptyPage
from django.db.models.fields.files import FieldFile
from django.views.generic import FormView
from django.views.generic.base import TemplateView

from django2_propeller.views import NavBarMixin
from .navbars import MainNavBar, DemoNavBar1, DemoNavBar2
from .forms import ContactForm, FilesForm, ContactFormSet
from .cards import DemoCard1, DemoCard2, DemoCard3, DemoCard4


# http://yuji.wordpress.com/2013/01/30/django-form-field-in-initial-data-requires-a-fieldfile-instance/
class FakeField(object):
    storage = default_storage


fieldfile = FieldFile(None, FakeField, 'dummy.txt')


class HomePageView(TemplateView, NavBarMixin):
    template_name = 'home.html'
    navbar_class = MainNavBar

    def get_context_data(self, **kwargs):
        context = super(HomePageView, self).get_context_data(**kwargs)
        messages.info(self.request, 'hello http://example.com')
        return context


class DefaultFormsetView(FormView, NavBarMixin):
    template_name = 'formset.html'
    form_class = ContactFormSet
    navbar_class = MainNavBar


class DefaultFormView(FormView, NavBarMixin):
    template_name = 'form.html'
    form_class = ContactForm
    navbar_class = MainNavBar


class DefaultFormByFieldView(FormView, NavBarMixin):
    template_name = 'form_by_field.html'
    form_class = ContactForm
    navbar_class = MainNavBar


class FormHorizontalView(FormView, NavBarMixin):
    template_name = 'form_horizontal.html'
    form_class = ContactForm
    navbar_class = MainNavBar


class FormInlineView(FormView, NavBarMixin):
    template_name = 'form_inline.html'
    form_class = ContactForm
    navbar_class = MainNavBar


class FormWithFilesView(FormView, NavBarMixin):
    template_name = 'form_with_files.html'
    form_class = FilesForm
    navbar_class = MainNavBar

    def get_context_data(self, **kwargs):
        context = super(FormWithFilesView, self).get_context_data(**kwargs)
        context['layout'] = self.request.GET.get('layout', 'vertical')
        return context

    def get_initial(self):
        return {
            'file4': fieldfile,
        }


class PaginationView(TemplateView, NavBarMixin):
    template_name = 'pagination.html'
    navbar_class = MainNavBar

    def get_context_data(self, **kwargs):
        context = super(PaginationView, self).get_context_data(**kwargs)
        lines = []
        for i in range(200):
            lines.append('Line %s' % (i + 1))
        paginator = Paginator(lines, 10)
        page = self.request.GET.get('page')
        try:
            show_lines = paginator.page(page)
        except PageNotAnInteger:
            # If page is not an integer, deliver first page.
            show_lines = paginator.page(1)
        except EmptyPage:
            # If page is out of range (e.g. 9999), deliver last page of results.
            show_lines = paginator.page(paginator.num_pages)
        context['lines'] = show_lines
        return context


class MiscView(TemplateView, NavBarMixin):
    template_name = 'misc.html'
    navbar_class = MainNavBar


class ButtonsView(TemplateView, NavBarMixin):
    template_name = 'buttons.html'
    navbar_class = MainNavBar


class FABsView(TemplateView, NavBarMixin):
    template_name = 'fabs.html'
    navbar_class = MainNavBar


class TypoView(TemplateView, NavBarMixin):
    template_name = 'typo.html'
    navbar_class = MainNavBar

    def get_context_data(self, **kwargs):
        context = super(TypoView, self).get_context_data(**kwargs)
        context['text1'] = "Propeller Heading"
        context['text2'] = "with secondary heading"
        context['text3'] = "Really large heading"
        context['text4'] = "Larger heading"
        context['text5'] = "Large heading"
        context['text6'] = "Normal heading"
        context['text7'] = "Heading"
        context['text8'] = "Lead text..."
        context['text9'] = "Normal text..."
        context['text10'] = "With this filter you can "
        context['text11'] = "highlight some text"
        context['text12'] = "strikethrough some text"
        context['text13'] = "underline some text"
        context['text14'] = "show some bold text"
        context['text15'] = "show some italic text"
        return context


class NavBarView(TemplateView, NavBarMixin):
    template_name = 'navbar.html'
    navbar_class = MainNavBar

    def get_context_data(self, **kwargs):
        context = super(NavBarView, self).get_context_data(**kwargs)
        context['navbar1'] = DemoNavBar1
        context['navbar2'] = DemoNavBar2
        return context


class CardView(TemplateView, NavBarMixin):
    template_name = 'cards.html'
    navbar_class = MainNavBar

    def get_context_data(self, **kwargs):
        context = super(CardView, self).get_context_data(**kwargs)
        context['card1'] = DemoCard1
        context['card2'] = DemoCard2
        context['card3'] = DemoCard3
        context['card4'] = DemoCard4
        return context



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import re
from math import floor

from django import template
from django.contrib.messages import constants as message_constants
from django.template import Context
from django.utils.html import conditional_escape
from django.utils.safestring import mark_safe

from ..components import render_icon, render_alert, render_bootstrap_icon
from ..forms import (
    render_button, render_field, render_form,
    render_formset, render_fab,
    render_label, render_form_errors, render_formset_errors
)
from ..propeller import (
    css_url, javascript_url, jquery_url, theme_url, get_propeller_setting
)
from ..text import force_text
from ..utils import url_replace_param
from ..utils import render_link_tag, render_tag, render_template_file

MESSAGE_LEVEL_CLASSES = {
    message_constants.DEBUG: "alert alert-warning",
    message_constants.INFO: "alert alert-info",
    message_constants.SUCCESS: "alert alert-success",
    message_constants.WARNING: "alert alert-warning",
    message_constants.ERROR: "alert alert-danger",
}

register = template.Library()


@register.filter
def propeller_setting(value):
    """
    A simple way to read Propeller settings in a template.

    Please consider this filter private for now, do not use it in your own
    templates.
    """
    return get_propeller_setting(value)


@register.filter
def propeller_message_classes(message):
    """Return the message classes for a message"""
    extra_tags = None
    try:
        extra_tags = message.extra_tags
    except AttributeError:
        pass
    if not extra_tags:
        extra_tags = ""
    classes = [extra_tags]
    try:
        level = message.level
    except AttributeError:
        pass
    else:
        try:
            classes.append(MESSAGE_LEVEL_CLASSES[level])
        except KeyError:
            classes.append("alert alert-danger")
    return ' '.join(classes).strip()


@register.simple_tag
def propeller_jquery_url():
    """
    Return the full url to jQuery file to use

    Default value: ``//code.jquery.com/jquery.min.js``

    This value is configurable, see Settings section

    **Tag name**::

        propeller_jquery_url

    **Usage**::

        {% propeller_jquery_url %}

    **Example**::

        {% propeller_jquery_url %}
    """
    return jquery_url()


@register.simple_tag
def propeller_javascript_url():
    """
    Return the full url to the Propeller JavaScript library

    Default value: ``None``

    This value is configurable, see Settings section

    **Tag name**::

        propeller_javascript_url

    **Usage**::

        {% propeller_javascript_url %}

    **Example**::

        {% propeller_javascript_url %}
    """
    return javascript_url()


@register.simple_tag
def propeller_css_url():
    """
    Return the full url to the Propeller CSS library

    Default value: ``None``

    This value is configurable, see Settings section

    **Tag name**::

        propeller_css_url

    **Usage**::

        {% propeller_css_url %}

    **Example**::

        {% propeller_css_url %}
    """
    return css_url()


@register.simple_tag
def propeller_theme_url():
    """
    Return the full url to a Propeller theme CSS library

    Default value: ``None``

    This value is configurable, see Settings section

    **Tag name**::

        propeller_theme_url

    **Usage**::

        {% propeller_theme_url %}

    **Example**::

        {% propeller_theme_url %}
    """
    return theme_url()


@register.simple_tag
def propeller_css():
    """
    Return HTML for Propeller CSS.

    Adjust url in settings. If no url is returned, we don't want this statement
    to return any HTML. This is intended behavior.

    Default value: ``None``

    This value is configurable, see Settings section

    **Tag name**::

        propeller_css

    **Usage**::

        {% propeller_css %}

    **Example**::

        {% propeller_css %}
    """
    rendered_urls = [render_link_tag(propeller_css_url()), ]
    if propeller_theme_url():
        rendered_urls.append(render_link_tag(propeller_theme_url()))
    return mark_safe(''.join([url for url in rendered_urls]))


@register.simple_tag
def propeller_javascript(jquery=None):
    """
    Return HTML for Propeller JavaScript.

    Adjust url in settings. If no url is returned, we don't want this
    statement to return any HTML.
    This is intended behavior.

    Default value: ``None``

    This value is configurable, see Settings section

    **Tag name**::

        propeller_javascript

    **Parameters**:

        :jquery: Truthy to include jQuery as well as propeller

    **Usage**::

        {% propeller_javascript %}

    **Example**::

        {% propeller_javascript jquery=1 %}
    """
    javascript = ''
    # See if we have to include jQuery
    if jquery is None:
        jquery = get_propeller_setting('include_jquery', False)
    # NOTE: No async on scripts, not mature enough. See issue #52 and #56
    if jquery:
        url = propeller_jquery_url()
        if url:
            javascript += render_tag('script', attrs={'src': url})
    url = propeller_javascript_url()
    if url:
        attrs = {'src': url}
        javascript += render_tag('script', attrs=attrs)
    return mark_safe(javascript)


@register.simple_tag
def propeller_formset(*args, **kwargs):
    """
    Render a formset

    **Tag name**::

        propeller_formset

    **Parameters**:

        formset
            The formset that is being rendered


        See propeller_field_ for other arguments

    **Usage**::

        {% propeller_formset formset %}

    **Example**::

        {% propeller_formset formset layout='horizontal' %}

    """
    return render_formset(*args, **kwargs)


@register.simple_tag
def propeller_formset_errors(*args, **kwargs):
    """
    Render formset errors

    **Tag name**::

        propeller_formset_errors

    **Parameters**:

        formset
            The formset that is being rendered

        layout
            Context value that is available in the template ``propeller/form_errors.html`` as ``layout``.

    **Usage**::

        {% propeller_formset_errors formset %}

    **Example**::

        {% propeller_formset_errors formset layout='inline' %}
    """
    return render_formset_errors(*args, **kwargs)


@register.simple_tag
def propeller_form(*args, **kwargs):
    """
    Render a form

    **Tag name**::

        propeller_form

    **Parameters**:

        form
            The form that is to be rendered

        exclude
            A list of field names (comma separated) that should not be rendered
            E.g. exclude=subject,bcc

        See propeller_field_ for other arguments

    **Usage**::

        {% propeller_form form %}

    **Example**::

        {% propeller_form form layout='inline' %}
    """
    return render_form(*args, **kwargs)


@register.simple_tag
def propeller_form_errors(*args, **kwargs):
    """
    Render form errors

    **Tag name**::

        propeller_form_errors

    **Parameters**:

        form
            The form that is to be rendered

        type
            Control which type of errors should be rendered.

            One of the following values:

                * ``'all'``
                * ``'fields'``
                * ``'non_fields'``

            :default: ``'all'``

        layout
            Context value that is available in the template ``propeller/form_errors.html`` as ``layout``.

    **Usage**::

        {% propeller_form_errors form %}

    **Example**::

        {% propeller_form_errors form layout='inline' %}
    """
    return render_form_errors(*args, **kwargs)


@register.simple_tag
def propeller_field(*args, **kwargs):
    """
    Render a field

    **Tag name**::

        propeller_field

    **Parameters**:


        field
            The form field to be rendered

        layout
            If set to ``'horizontal'`` then the field and label will be rendered side-by-side, as long as there
            is no ``field_class`` set as well.

        form_group_class
            CSS class of the ``div`` that wraps the field and label.

            :default: ``'form-group'``

        field_class
            CSS class of the ``div`` that wraps the field.

        label_class
            CSS class of the ``label`` element. Will always have ``control-label`` as the last CSS class.

        show_help
            Show the field's help text, if the field has help text.

            :default: ``True``

        show_label
            Whether the show the label of the field.

            :default: ``True``

        exclude
            A list of field names that should not be rendered

        set_required
            When set to ``True`` and the field is required then the ``required`` attribute is set on the
            rendered field. This only works up to Django 1.8. Higher Django versions handle ``required``
            natively.

            :default: ``True``

        set_disabled
            When set to ``True`` then the ``disabled`` attribute is set on the rendered field. This only
            works up to Django 1.8.  Higher Django versions handle ``disabled`` natively.

            :default: ``False``

        size
            Controls the size of the rendered ``div.form-group`` through the use of CSS classes.

            One of the following values:

                * ``'small'``
                * ``'medium'``
                * ``'large'``

        placeholder
            Sets the placeholder text of a textbox

        horizontal_label_class
            Class used on the label when the ``layout`` is set to ``horizontal``.

            :default: ``'col-md-3'``. Can be changed in :doc:`settings`

        horizontal_field_class
            Class used on the field when the ``layout`` is set to ``horizontal``.

            :default: ``'col-md-9'``. Can be changed in :doc:`settings`

        addon_before
            Text that should be prepended to the form field.
            See the `propeller docs <http://getpropeller.com/components/#input-groups-basic>` for an example.

        addon_after
            Text that should be appended to the form field.
            See the `propeller docs <http://getpropeller.com/components/#input-groups-basic>` for an example.

        addon_before_class
            Class used on the span when ``addon_before`` is used.

            One of the following values:
                
                * ``'input-group-addon'``
                * ``'input-group-btn'``

            :default: ``input-group-addon``

        addon_after_class
            Class used on the span when ``addon_after`` is used.

            One of the following values:
                
                * ``'input-group-addon'``
                * ``'input-group-btn'``

            :default: ``input-group-addon``

        error_css_class
            CSS class used when the field has an error

            :default: ``'has-error'``. Can be changed :doc:`settings`

        required_css_class
            CSS class used on the ``div.form-group`` to indicate a field is required

            :default: ``''``. Can be changed :doc:`settings`

        bound_css_class
            CSS class used when the field is bound

            :default: ``'has-success'``. Can be changed :doc:`settings`

    **Usage**::

        {% propeller_field field %}

    **Example**::

        {% propeller_field field show_label=False %}
    """
    return render_field(*args, **kwargs)


@register.simple_tag()
def propeller_label(*args, **kwargs):
    """
    Render a label

    **Tag name**::

        propeller_label

    **Parameters**:

        content
            The label's text

        label_for
            The value that will be in the ``for`` attribute of the rendered ``<label>``

        label_class
            The CSS class for the rendered ``<label>``

        label_title
            The value that will be in the ``title`` attribute of the rendered ``<label>``

    **Usage**::

        {% propeller_label content %}

    **Example**::

        {% propeller_label "Email address" label_for="exampleInputEmail1" %}

    """
    return render_label(*args, **kwargs)


@register.simple_tag
def propeller_button(*args, **kwargs):
    """
    Render a button

    **Tag name**::

        propeller_button

    **Parameters**:

        content
            The text to be displayed in the button

        button_type
            Optional field defining what type of button this is.

            Accepts one of the following values:

                * ``'submit'``
                * ``'reset'``
                * ``'button'``
                * ``'link'``

        style
            Optional field defining which style button should have.

            Accepts one of the following values:

                * ``'default'``
                * ``'raised'``
                * ``'flat'``
                * ``'outline'``

        icon
            Name of an icon to render in the button's visible content. See propeller_icon_ for acceptable values.

        button_class
            The class of button to use. If none is given, btn-default will be used.

        extra_classes
            Any extra CSS classes that should be added to the button.

        size
            Optional field to control the size of the button.

            Accepts one of the following values:

                * ``'xs'``
                * ``'sm'``
                * ``'small'``
                * ``'md'``
                * ``'medium'``
                * ``'lg'``
                * ``'large'``


        href
            Render the button as an ``<a>`` element. The ``href`` attribute is set with this value.

        name
            Value of the ``name`` attribute of the rendered element.

        value
            Value of the ``value`` attribute of the rendered element.

    **Usage**::

        {% propeller_button content %}

    **Example**::

        {% propeller_button "Save" button_type="submit" button_class="btn-primary" %}
    """
    return render_button(*args, **kwargs)


@register.simple_tag
def propeller_fab(*args, **kwargs):
    """
    Render a floating action button

    **Tag name**::

        propeller_fab

    **Parameters**:

        content
            The text to be displayed in the button

        button_type
            Optional field defining what type of button this is.

            Accepts one of the following values:

                * ``'submit'``
                * ``'reset'``
                * ``'button'``
                * ``'link'``

        style
            Optional field defining which style button should have.

            Accepts one of the following values:

                * ``'default'``
                * ``'raised'``
                * ``'flat'``
                * ``'outline'``

        icon
            Name of an icon to render in the button's visible content. See propeller_icon_ for acceptable values.

        button_class
            The class of button to use. If none is given, btn-default will be used.

        extra_classes
            Any extra CSS classes that should be added to the button.

        size
            Optional field to control the size of the button.

            Accepts one of the following values:

                * ``'xs'``
                * ``'sm'``
                * ``'small'``
                * ``'md'``
                * ``'medium'``
                * ``'lg'``
                * ``'large'``


        href
            Render the button as an ``<a>`` element. The ``href`` attribute is set with this value.

        name
            Value of the ``name`` attribute of the rendered element.

        value
            Value of the ``value`` attribute of the rendered element.

    **Usage**::

        {% propeller_fab content %}

    **Example**::

        {% propeller_fab "Save" button_type="submit" button_class="btn-primary" %}
    """
    return render_fab(*args, **kwargs)


@register.simple_tag
def propeller_icon(icon, **kwargs):
    """
    Render an icon

    **Tag name**:

        propeller_icon

    **Parameters**:

        icon
            Icon name. See the `Propeller docs <http://propeller.in/style/icons.php>`_ for all icons.

        size
            Size of the icon. Must be one of 'xs', 'sm', 'md', or 'lg'. Default: 'sm'

        extra_classes
            Extra CSS classes to add to the icon HTML. Optional

        title
            A title for the icon (HTML title attrivute). Optional

    **Usage**::

        {% propeller_icon icon %}

    **Example**::

        {% propeller_icon "star" %}

    """
    return render_icon(icon, **kwargs)


@register.simple_tag
def propeller_bootstrap_icon(icon, **kwargs):
    """
    Render an icon

    **Tag name**:

        propeller_bootstrap_icon

    **Parameters**:

        icon
            Icon name. See the `Bootstrap docs <http://getbootstrap.com/components/#glyphicons>`_ for all icons.

        extra_classes
            Extra CSS classes to add to the icon HTML. Optional

        title
            A title for the icon (HTML title attrivute). Optional

    **Usage**::

        {% propeller_bootstrap_icon icon %}

    **Example**::

        {% propeller_bootstrap_icon "star" %}

    """
    return render_bootstrap_icon(icon, **kwargs)


@register.simple_tag
def propeller_alert(content, alert_type='info', dismissable=True):
    """
    Render an alert

    **Tag name**::

        propeller_alert

    **Parameters**:

        content
            HTML content of alert

        alert_type
            * ``'info'``
            * ``'warning'``
            * ``'danger'``
            * ``'success'``

            :default: ``'info'``

        dismissable
            boolean, is alert dismissable

            :default: ``True``

    **Usage**::

        {% propeller_alert content %}

    **Example**::

        {% propeller_alert "Something went wrong" alert_type='error' %}

    """
    return render_alert(content, alert_type, dismissable)


@register.simple_tag(takes_context=True)
def propeller_messages(context, *args, **kwargs):
    """
    Show django.contrib.messages Messages in propeller alert containers.

    In order to make the alerts dismissable (with the close button),
    we have to set the jquery parameter too when using the
    propeller_javascript tag.

    Uses the template ``propeller/messages.html``.

    **Tag name**::

        propeller_messages

    **Parameters**:

        None.

    **Usage**::

        {% propeller_messages %}

    **Example**::

        {% propeller_javascript jquery=1 %}
        {% propeller_messages %}

    """
    # Force Django 1.8+ style, so dicts and not Context
    # TODO: This may be due to a bug in Django 1.8/1.9+
    if Context and isinstance(context, Context):
        context = context.flatten()
    context.update({'message_constants': message_constants})
    return render_template_file('propeller/messages.html', context=context)


@register.inclusion_tag('propeller/pagination.html')
def propeller_pagination(page, **kwargs):
    """
    Render pagination for a page

    **Tag name**::

        propeller_pagination

    **Parameters**:

        page
            The page of results to show.

        pages_to_show
            Number of pages in total

            :default: ``11``

        url
            URL to navigate to for pagination forward and pagination back.

            :default: ``None``

        size
            Controls the size of the pagination through CSS. Defaults to being normal sized.

            One of the following:

                * ``'small'``
                * ``'large'``

            :default: ``None``

        extra
            Any extra page parameters.

            :default: ``None``

        parameter_name
            Name of the paging URL parameter.

            :default: ``'page'``

    **Usage**::

        {% propeller_pagination page %}

    **Example**::

        {% propeller_pagination lines url="/pagination?page=1" size="large" %}

    """
    pagination_kwargs = kwargs.copy()
    pagination_kwargs['page'] = page
    return get_pagination_context(**pagination_kwargs)


@register.simple_tag
def propeller_url_replace_param(url, name, value):
    return url_replace_param(url, name, value)


def get_pagination_context(page, pages_to_show=11,
                           url=None, size=None, extra=None,
                           parameter_name='page'):
    """Generate propeller pagination context from a page object"""
    pages_to_show = int(pages_to_show)
    if pages_to_show < 1:
        raise ValueError(
            "Pagination pages_to_show should be a positive integer, you specified {pages}".format(
                pages=pages_to_show)
        )
    num_pages = page.paginator.num_pages
    current_page = page.number
    half_page_num = int(floor(pages_to_show / 2))
    if half_page_num < 0:
        half_page_num = 0
    first_page = current_page - half_page_num
    if first_page <= 1:
        first_page = 1
    if first_page > 1:
        pages_back = first_page - half_page_num
        if pages_back < 1:
            pages_back = 1
    else:
        pages_back = None
    last_page = first_page + pages_to_show - 1
    if pages_back is None:
        last_page += 1
    if last_page > num_pages:
        last_page = num_pages
    if last_page < num_pages:
        pages_forward = last_page + half_page_num
        if pages_forward > num_pages:
            pages_forward = num_pages
    else:
        pages_forward = None
        if first_page > 1:
            first_page -= 1
        if pages_back is not None and pages_back > 1:
            pages_back -= 1
        else:
            pages_back = None
    pages_shown = []
    for i in range(first_page, last_page + 1):
        pages_shown.append(i)
        # Append proper character to url
    if url:
        # Remove existing page GET parameters
        url = force_text(url)
        url = re.sub(r'\?{0}\=[^\&]+'.format(parameter_name), '?', url)
        url = re.sub(r'\&{0}\=[^\&]+'.format(parameter_name), '', url)
        # Append proper separator
        if '?' in url:
            url += '&'
        else:
            url += '?'
            # Append extra string to url
    if extra:
        if not url:
            url = '?'
        url += force_text(extra) + '&'
    if url:
        url = url.replace('?&', '?')
    # Set CSS classes, see http://getpropeller.com/components/#pagination
    pagination_css_classes = ['pagination']
    if size == 'small':
        pagination_css_classes.append('pagination-sm')
    elif size == 'large':
        pagination_css_classes.append('pagination-lg')
        # Build context object
    return {
        'propeller_pagination_url': url,
        'num_pages': num_pages,
        'current_page': current_page,
        'first_page': first_page,
        'last_page': last_page,
        'pages_shown': pages_shown,
        'pages_back': pages_back,
        'pages_forward': pages_forward,
        'pagination_css_classes': ' '.join(pagination_css_classes),
        'parameter_name': parameter_name,
    }


@register.filter(is_safe=True)
def pmd_muted_text(text):
    """
    Render a muted text (secondary heading).

    **Tag name**::

        pmd_muted_text

    **Usage**::

        {{ text_variable|pmd_muted_text }}
    """
    esc = conditional_escape
    result = '<span class="text-muted">%s</span>' % esc(text)
    return mark_safe(result)


@register.filter(is_safe=True)
def pmd_display_text(text, size=1):
    """
    Render text as a Propeller display text (heading).

    **Tag name**::

        pmd_display_text

    **Parameters**::

        size
            Controls the size of the heading.

            An integer from 1 (normal) to 4 (very large)

            :default: ``1``

    **Usage**::

        {{ text_variable|pmd_display_text:size }}

    **Example**::

        {{ my_text|pmd_display_text:3 }}
    """
    esc = conditional_escape
    result = '<span class="pmd-display%d">%s</span>' % (int(size), esc(text))
    return mark_safe(result)


@register.filter(is_safe=True)
def pmd_lead_text(text):
    """
    Render text as a Propeller lead text (intro).

    **Tag name**::

        pmd_lead_text

    **Usage**::

        {{ text_variable|pmd_lead_text }}
    """
    esc = conditional_escape
    result = '<span class="lead">%s</span>' % esc(text)
    return mark_safe(result)


@register.filter(is_safe=True)
def pmd_mark_text(text):
    """
    Render highligthed text.

    **Tag name**::

        pmd_mark_text

    **Usage**::

        {{ text_variable|pmd_mark_text }}
    """
    esc = conditional_escape
    result = '<mark>%s</mark>' % esc(text)
    return mark_safe(result)


@register.filter(needs_autoescape=True)
def pmd_strike_text(text, autoescape=True):
    """
    Render striked text.

    **Tag name**::

        pmd_strike_text

    **Usage**::

        {{ text_variable|pmd_strike_text }}
    """
    if autoescape:
        esc = conditional_escape
    else:
        esc = lambda x: x
    result = '<s>%s</s>' % esc(text)
    return mark_safe(result)


@register.filter(is_safe=True)
def pmd_underline_text(text):
    """
    Render underlined text.

    **Tag name**::

        pmd_underline_text

    **Usage**::

        {{ text_variable|pmd_underline_text }}
    """
    esc = conditional_escape
    result = '<u>%s</u>' % esc(text)
    return mark_safe(result)


@register.filter(is_safe=True)
def pmd_bold_text(text):
    """
    Render bold text.

    **Tag name**::

        pmd_bold_text

    **Usage**::

        {{ text_variable|pmd_bold_text }}
    """
    esc = conditional_escape
    result = '<strong>%s</strong>' % esc(text)
    return mark_safe(result)


@register.filter(is_safe=True)
def pmd_italic_text(text):
    """
    Render italic text.

    **Tag name**::

        pmd_italic_text

    **Usage**::

        {{ text_variable|pmd_italic_text }}
    """
    esc = conditional_escape
    result = '<em>%s</em>' % esc(text)
    return mark_safe(result)


@register.simple_tag
def propeller_navbar(navbar):
    """
    Render a navbar.

    **Tag name**::

        propeller_navbar

    **Parameters**:

        navbar
            The previously defined navbar instance

    **Usage**::

        {% propeller_navbar navbar_instance %}
    """
    return navbar.as_html()


@register.simple_tag
def propeller_card(card):
    """
    Render a propeller card.

    **Tag name**::

        propeller_card

    **Parameters**:

        card
            The previously defined card instance

    **Usage**::

        {% propeller_card card_instance %}
    """
    return card.as_html()



# -*- coding: utf-8 -*-

"""This module contains classes for constructing propeller cards"""

from django.utils.safestring import mark_safe

from .utils import render_tag, add_css_class
from .components import Button, FAB, Image
from .text import text_concat
from .exceptions import PropellerException


class CardTitle(object):

    """
    Renders a Card Title.

    **Parameters**:

        text
            The display text for the title.

        size
            The size for the title as integer. Works with the h-tag, so size=1 is bigger than size=3.
            Optional. (default=3)
    """

    text = ""
    size = 3

    def as_html(self):
        """Returns card title as html"""
        tag = 'h%d' % self.size
        attrs = {'class': 'pmd-card-title-text'}
        content = self.text
        return render_tag(tag, attrs=attrs, content=mark_safe(content), )


class CardSubtitle(object):

    """
    Renders a Card Subtitle.

    **Parameters**:

        text
            The display text for the subtitle.
    """

    text = ""

    def as_html(self):
        """Returns card subtitle as html"""
        tag = 'span'
        attrs = {'class': 'pmd-card-subtitle-text'}
        content = self.text
        return render_tag(tag, attrs=attrs, content=mark_safe(content), )


class CardBody(object):

    """
    Renders a Card Body.

    **Parameters**:

        text
            The text to display in the body.
    """

    text = ""

    def as_html(self):
        """Returns card body as html"""
        tag = 'div'
        attrs = {'class': 'pmd-card-body'}
        content = self.text
        return render_tag(tag, attrs=attrs, content=mark_safe(content), )


class CardHeader(object):

    """
    Renders a Card Header.

    **Parameters**:

        content_left
            A list of items to display on the left of header. 
            May contain Button, FAB, Image, CardTitle, or CardSubtitle.

        content_middle
            A list of items to display in the middle of header. 
            May contain Button, FAB, Image, CardTitle, or CardSubtitle.
    """

    content_left = []
    content_middle = []

    def get_left_content(self):
        """Returns left content of card header as html"""
        tag = 'div'
        attrs = {'class': 'media-left'}
        content = ''
        for itm in self.content_left:
            content = text_concat(content, mark_safe(itm.as_html()))
        return render_tag(tag, attrs=attrs, content=mark_safe(content), )

    def get_middle_content(self):
        """Returns middle content of card header as html"""
        tag = 'div'
        attrs = {'class': 'media-body media-middle'}
        content = ''
        for itm in self.content_middle:
            content = text_concat(content, mark_safe(itm.as_html()))
        return render_tag(tag, attrs=attrs, content=mark_safe(content), )

    def as_html(self):
        """Returns card header as html"""
        tag = 'div'
        attrs = {'class': 'pmd-card-title'}
        content = text_concat(self.get_left_content(), self.get_middle_content())
        return render_tag(tag, attrs=attrs, content=mark_safe(content), )


class CardMediaActions(object):

    """
    Renders Card Media Actions.

    **Parameters**:

        items
            A list of items to display in the Card Media Action section. 
            May contain Button or FAB.
    """

    items = []

    def as_html(self):
        """Returns card media actions as html"""
        tag = 'div'
        attrs = {'class': 'pmd-card-actions'}
        content = ''
        for btn in self.items:
            if isinstance(btn, FAB):
                content = text_concat(content, mark_safe(btn.as_html()))
        return render_tag(tag, attrs=attrs, content=mark_safe(content), )


class CardActions(object):

    """
    Renders Card Actions.

    **Parameters**:

        items
            A list of items to display in the Card Action section. 
            May contain Button or FAB.
    """

    items = []

    def as_html(self):
        """Returns card actions as html"""
        tag = 'div'
        attrs = {'class': 'pmd-card-actions'}
        content = ''
        for btn in self.items:
            if isinstance(btn, Button):
                content = text_concat(content, mark_safe(btn.as_html()))
        return render_tag(tag, attrs=attrs, content=mark_safe(content), )


class CardMediaImage(object):

    """
    Renders a Card Media Image.

    **Parameters**:

        image
            Must be an instance of an Image.
    """

    image = None

    def as_html(self):
        """Returns card media image as html"""
        if isinstance(self.image, Image):
            return self.image.as_html()
        return None


class CardMedia(object):

    """
    Renders Card Media

    **Parameters**:

        content
            if style_inline=True:
                A list of items to display in the card media section. 
                May contain CardMediaImage, CardTitle, or CardSubtitle.
            or if style_inline=False: (default)
                A instance of CardMediaImage

        style_inline
            Display card with inline style if true. (Default: False)
    """

    content = None
    style_inline = False

    def get_media_body_inline(self):
        """Returns media body inline as html"""
        tag = 'div'
        attrs = {'class': 'media-body'}
        content = ''
        if self.style_inline:
            if not isinstance(self.content, list):
                raise PropellerException("Propeller Card: content must be a list")
            for itm in self.content:
                if isinstance(itm, (CardTitle, CardSubtitle)):
                    content = text_concat(content, mark_safe(itm.as_html()))
            content = text_concat(content, mark_safe('</div>'))
            content = text_concat(content, mark_safe('<div class="media-right media-middle">'))
            for itm in self.content:
                if isinstance(itm, CardMediaImage):
                    content = text_concat(content, mark_safe(itm.as_html()))
        return render_tag(tag, attrs=attrs, content=mark_safe(content), )

    def get_media_body(self):
        """Returns media body as html"""
        if self.style_inline:
            return self.get_media_body_inline()

        tag = 'div'
        attrs = {'class': 'media-body'}
        content = ''
        if not isinstance(self.content, list):
            raise PropellerException("Propeller Card: content must be a list")
        for itm in self.content:
            if isinstance(itm, CardMediaImage):
                content = text_concat(content, mark_safe(itm.as_html()))

        return render_tag(tag, attrs=attrs, content=mark_safe(content), )

    def as_html(self):
        """Returns card media as html"""
        tag = 'div'
        attrs = {'class': 'pmd-card-media'}
        content = self.get_media_body()
        return render_tag(tag, attrs=attrs, content=mark_safe(content), )


class Card(object):

    """
    Card is a class that generates a Propeller Card.

    **Parameters**:

        primary_title
            An instance of CardTitle. Optional.

        secondary_title
            An instance of CardSubtitle. Optional.

        header
            An instance of CardHeader. Optional.

        media
            An instance of CardMedia. Optional.

        body
            An instance of CardBody. Optional.

        actions
            An instance of CardActions. Optional.

        media_actions
            An instance of CardMediaActions. Optional.

        style_inverse
            Display dark style if true. (Default: False)

        style_inline
            Display card with inline style if true. (Default: False)

        width
            Width of the card in Bootstrap grid (col-md) as integer. (Default: 4)
    """

    primary_title = None
    secondary_title = None
    header = None
    media = None
    body = None
    actions = None
    media_actions = None
    style_inverse = False
    style_inline = False
    width = 4

    def get_actions(self):
        """Returns actions of card as html"""
        actions = ''
        if self.media_actions:
            actions = text_concat(actions, self.media_actions.as_html())
        if self.actions:
            actions = text_concat(actions, self.actions.as_html())
        return actions

    def get_content(self):
        """Returns content of card as html"""
        content = ''
        if self.header and not self.style_inline:
            content = text_concat(content, self.header.as_html())
        if self.media:
            content = text_concat(content, self.media.as_html())
        if not self.style_inline:
            if self.primary_title or self.secondary_title:
                content = text_concat(content, '<div class="pmd-card-title">')
                if self.primary_title:
                    content = text_concat(content, self.primary_title.as_html())
                if self.secondary_title:
                    content = text_concat(content, self.secondary_title.as_html())
                content = text_concat(content, '</div>')
            if self.body:
                content = text_concat(content, self.body.as_html())
        content = text_concat(content, self.get_actions())
        return content

    def as_html(self):
        """Returns card as html"""
        tag = 'div'
        classes = 'pmd-card'
        if self.style_inline:
            classes = add_css_class(classes, 'pmd-card-media-inline')
        if self.style_inverse:
            classes = add_css_class(classes, 'pmd-card-inverse')
        else:
            classes = add_css_class(classes, 'pmd-card-default')
        classes = add_css_class(classes, 'pmd-z-depth')
        classes = add_css_class(classes, 'col-md-%d' % self.width)
        attrs = {'class': classes}
        content = self.get_content()
        return render_tag(tag, attrs=attrs, content=mark_safe(content), )



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import re
from collections import Mapping

try:  # pragma: no cover
    from urllib.parse import urlencode
except ImportError:  # pragma: no cover
    from urllib import urlencode

try:  # pragma: no cover
    from urllib.parse import urlparse, parse_qs, urlunparse
except ImportError:  # pragma: no cover
    from urlparse import urlparse, parse_qs, urlunparse

from django.forms.utils import flatatt
from django.template.loader import get_template
from django.utils.encoding import force_str, force_text
from django.utils.safestring import mark_safe
from django.utils.html import format_html

from .text import text_value

# RegEx for quoted string
QUOTED_STRING = re.compile(r'^["\'](?P<noquotes>.+)["\']$')


def split_css_classes(css_classes):
    """Turn string into a list of CSS classes"""
    classes_list = text_value(css_classes).split(' ')
    return [c for c in classes_list if c]


def add_css_class(css_classes, css_class, prepend=False):
    """Add a CSS class to a string of CSS classes"""
    classes_list = split_css_classes(css_classes)
    classes_to_add = [c for c in split_css_classes(css_class)
                      if c not in classes_list]
    if prepend:
        classes_list = classes_to_add + classes_list
    else:
        classes_list += classes_to_add
    return ' '.join(classes_list)


def render_link_tag(url, rel='stylesheet', media=None):
    """Build a link tag"""
    attrs = {
        'href': url,
        'rel': rel,
    }
    if media:
        attrs['media'] = media
    return render_tag('link', attrs=attrs, close=False)


def render_tag(tag, attrs=None, content=None, close=True):
    """Render a HTML tag"""
    builder = '<{tag}{attrs}>{content}'
    if content or close:
        builder += '</{tag}>'
    return format_html(
        builder,
        tag=tag,
        attrs=mark_safe(flatatt(attrs)) if attrs else '',
        content=text_value(content),
    )


def render_template_file(template, context=None):
    """Render a Template to unicode"""
    assert isinstance(context, Mapping)
    template = get_template(template)
    return template.render(context)


def url_replace_param(url, name, value):
    """Replace a GET parameter in an URL"""
    url_components = urlparse(force_str(url))
    query_params = parse_qs(url_components.query)
    query_params[name] = value
    query = urlencode(query_params, doseq=True)
    return force_text(urlunparse([
        url_components.scheme,
        url_components.netloc,
        url_components.path,
        url_components.params,
        query,
        url_components.fragment,
    ]))



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.contrib.admin.widgets import AdminFileWidget
from django.forms import (
    HiddenInput, FileInput, CheckboxSelectMultiple, Textarea, TextInput,
    PasswordInput
)
from django.forms.widgets import CheckboxInput
from django.utils.safestring import mark_safe

from .propeller import (
    get_propeller_setting, get_form_renderer, get_field_renderer,
    get_formset_renderer,
    PROPELLER_SET_REQUIRED_SET_DISABLED
)
from .components import Button, FAB
from .utils import add_css_class, render_tag

FORM_GROUP_CLASS = 'form-group pmd-textfield'


def render_formset(formset, **kwargs):
    """
    Render a formset to a Bootstrap layout
    """
    renderer_cls = get_formset_renderer(**kwargs)
    return renderer_cls(formset, **kwargs).render()


def render_formset_errors(formset, **kwargs):
    """
    Render formset errors to a Bootstrap layout
    """
    renderer_cls = get_formset_renderer(**kwargs)
    return renderer_cls(formset, **kwargs).render_errors()


def render_form(form, **kwargs):
    """Render a form to a Bootstrap layout"""
    renderer_cls = get_form_renderer(**kwargs)
    return renderer_cls(form, **kwargs).render()


def render_form_errors(form, _type='all', **kwargs):
    """Render form errors to a Bootstrap layout"""
    renderer_cls = get_form_renderer(**kwargs)
    return renderer_cls(form, **kwargs).render_errors(_type)


def render_field(field, **kwargs):
    """Render a field to a Bootstrap layout"""
    renderer_cls = get_field_renderer(**kwargs)
    return renderer_cls(field, **kwargs).render()


def render_label(content, label_for=None, label_class=None, label_title=''):
    """Render a label with content"""
    attrs = {}
    if label_for:
        attrs['for'] = label_for
    if label_class:
        attrs['class'] = label_class
    if label_title:
        attrs['title'] = label_title
    return render_tag('label', attrs=attrs, content=content)


def render_button(content, button_type=None, icon=None, button_class='btn-default', size='',
                  href='', name=None, value=None, title=None, style='default', extra_classes='', _id=''):
    """Render a button with content"""
    return Button(content, button_type, icon, button_class, size, href, name, value, title,
                  style, extra_classes, _id).as_html()


def render_fab(content, button_type=None, icon=None, button_class='btn-default', size='', href='',
               name=None, value=None, title=None, style='default', extra_classes='', _id=''):
    """Render a button with content"""
    return FAB(content, button_type, icon, button_class, size, href, name, value, title,
               style, extra_classes, _id).as_html()


def render_field_and_label(
        field, label, field_class='', label_for=None, label_class='',
        layout='', **kwargs):
    """Render a field with its label"""
    if layout == 'horizontal':
        if not label_class:
            label_class = get_propeller_setting('horizontal_label_class')
        if not field_class:
            field_class = get_propeller_setting('horizontal_field_class')
        if not label:
            label = mark_safe('&#160;')
        label_class = add_css_class(label_class, 'control-label')
    html = field
    if field_class:
        html = '<div class="{klass}">{html}</div>'.format(
            klass=field_class, html=html)
    if label:
        html = render_label(
            label, label_for=label_for, label_class=label_class) + html
    return html


def render_form_group(content, css_class=FORM_GROUP_CLASS):
    """Render a Bootstrap form group"""
    return '<div class="{klass}">{content}</div>'.format(
        klass=css_class,
        content=content,
    )


def is_widget_required_attribute(widget):
    """Is this widget required?"""
    if PROPELLER_SET_REQUIRED_SET_DISABLED and not get_propeller_setting('set_required'):
        return False
    if not widget.is_required:
        return False
    if isinstance(
            widget, (
                    AdminFileWidget, HiddenInput, FileInput,
                    CheckboxInput, CheckboxSelectMultiple)):
        return False
    return True


def is_widget_with_placeholder(widget):
    """
    Is this a widget that should have a placeholder?
    Only text, search, url, tel, e-mail, password, number have placeholders
    These are all derived form TextInput, except for Textarea
    """
    # PasswordInput inherits from Input in Django 1.4.
    # It was changed to inherit from TextInput in 1.5.
    return isinstance(widget, (TextInput, Textarea, PasswordInput))



from django2_propeller.card import Card, CardHeader, CardActions, CardMediaActions, CardTitle, CardSubtitle, \
    CardMediaImage, CardBody, CardMedia
from django2_propeller.components import Image, Button, FAB


class DemoTitle1(CardTitle):
    text = "Title goes here"
    size = 2


class DemoHeaderTitle1(CardTitle):
    text = "Two-line item"
    size = 3


class DemoSubtitle1(CardSubtitle):
    text = "Secondary text"


class DemoMediaImage1(CardMediaImage):
    image = Image(source="http://propeller.in/assets/images/profile-pic.png", responsive=True)


class DemoMediaImage2(CardMediaImage):
    image = Image(source="http://propeller.in/assets/images/profile-pic.png", width=80, height=80)


class DemoMedia1(CardMedia):
    content = [DemoMediaImage1(), ]


class DemoMedia2(CardMedia):
    content = [DemoTitle1(), DemoSubtitle1(), DemoMediaImage2(), ]


class DemoBody1(CardBody):
    text = "Cards provide context and an entry point to more robust information and views. " \
           "Don't overload cards with extraneous information or actions."


class DemoHeader1(CardHeader):
    content_middle = [DemoHeaderTitle1(), DemoSubtitle1()]
    content_left = [Image("http://propeller.in/assets/images/avatar-icon-40x40.png", width=40, height=40, avatar=True)]


class DemoActions1(CardActions):
    items = [
        Button(content='primary', button_class='btn-primary'),
        Button('Action'),
        Button('third')
    ]


class DemoMediaActions1(CardMediaActions):
    items = [
        FAB('', button_class='btn-primary', icon='share', style='flat', size='sm'),
        FAB('', button_class='btn-primary', icon='thumb_up', style='flat', size='sm'),
        FAB('', button_class='btn-primary', icon='drafts', style='flat', size='sm')
    ]


class DemoCard1(Card):
    primary_title = DemoTitle1()
    secondary_title = DemoSubtitle1()
    header = DemoHeader1()
    actions = DemoActions1()
    media_actions = DemoMediaActions1()
    media = DemoMedia1()
    body = DemoBody1()


class DemoCard2(DemoCard1):
    body = None
    header = None
    actions = None


class DemoCard3(DemoCard1):
    style_inverse = True


class DemoCard4(DemoCard1):
    style_inline = True
    body = None
    header = None
    media_actions = None
    media = DemoMedia2()



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

try:
    # If Django is set up without a database, importing this widget gives RuntimeError
    from django.contrib.auth.forms import ReadOnlyPasswordHashWidget
except RuntimeError:
    ReadOnlyPasswordHashWidget = None

from django.forms import (
    TextInput, DateInput, FileInput, CheckboxInput, MultiWidget,
    ClearableFileInput, Select, RadioSelect, CheckboxSelectMultiple
)
from django.forms.widgets import SelectDateWidget
from django.forms.forms import BaseForm, BoundField
from django.forms.formsets import BaseFormSet
from django.utils.html import conditional_escape, escape, strip_tags
from django.utils.safestring import mark_safe

from .propeller import get_propeller_setting, PROPELLER_SET_REQUIRED_SET_DISABLED
from .exceptions import PropellerError
from .forms import (
    render_form, render_field, render_label, render_form_group,
    is_widget_with_placeholder, FORM_GROUP_CLASS,
    is_widget_required_attribute
)
from .text import text_value
from .utils import add_css_class, render_template_file


class BaseRenderer(object):
    """
    A content renderer
    """

    def __init__(self, *args, **kwargs):
        self.layout = kwargs.get('layout', '')
        self.form_group_class = kwargs.get(
            'form_group_class', FORM_GROUP_CLASS)
        self.field_class = kwargs.get('field_class', '')
        self.label_class = kwargs.get('label_class', '')
        self.show_help = kwargs.get('show_help', True)
        self.show_label = kwargs.get('show_label', True)
        self.exclude = kwargs.get('exclude', '')

        # Only relevant if DBS3_SET_REQUIRED_SET_DISABLED
        self.set_required = kwargs.get('set_required', True)
        self.set_disabled = kwargs.get('set_disabled', False)

        self.set_placeholder = kwargs.get('set_placeholder', True)
        self.size = self.parse_size(kwargs.get('size', ''))
        self.horizontal_label_class = kwargs.get(
            'horizontal_label_class',
            get_propeller_setting('horizontal_label_class')
        )
        self.horizontal_field_class = kwargs.get(
            'horizontal_field_class',
            get_propeller_setting('horizontal_field_class')
        )

    @staticmethod
    def parse_size(size):
        size = text_value(size).lower().strip()
        if size in ('sm', 'small'):
            return 'small'
        if size in ('lg', 'large'):
            return 'large'
        if size in ('md', 'medium', ''):
            return 'medium'
        raise PropellerError('Invalid value "%s" for parameter "size" (expected "sm", "md", "lg" or "").' % size)

    def get_size_class(self, prefix='input'):
        if self.size == 'small':
            return prefix + '-sm'
        if self.size == 'large':
            return prefix + '-lg'
        return ''

    def _render(self):
        return ''

    def render(self):
        return mark_safe(self._render())


class FormsetRenderer(BaseRenderer):
    """
    Default formset renderer
    """

    def __init__(self, formset, *args, **kwargs):
        if not isinstance(formset, BaseFormSet):
            raise PropellerError(
                'Parameter "formset" should contain a valid Django Formset.')
        self.formset = formset
        super(FormsetRenderer, self).__init__(*args, **kwargs)

    def render_management_form(self):
        return text_value(self.formset.management_form)

    @staticmethod
    def render_form(form, **kwargs):
        return render_form(form, **kwargs)

    def render_forms(self):
        rendered_forms = []
        for form in self.formset.forms:
            rendered_forms.append(self.render_form(
                form,
                layout=self.layout,
                form_group_class=self.form_group_class,
                field_class=self.field_class,
                label_class=self.label_class,
                show_label=self.show_label,
                show_help=self.show_help,
                exclude=self.exclude,
                set_required=self.set_required,  # Only relevant if DBS3_SET_REQUIRED_SET_DISABLED
                set_disabled=self.set_disabled,  # Only relevant if DBS3_SET_REQUIRED_SET_DISABLED
                set_placeholder=self.set_placeholder,
                size=self.size,
                horizontal_label_class=self.horizontal_label_class,
                horizontal_field_class=self.horizontal_field_class,
            ))
        return '\n'.join(rendered_forms)

    def get_formset_errors(self):
        return self.formset.non_form_errors()

    def render_errors(self):
        formset_errors = self.get_formset_errors()
        if formset_errors:
            return render_template_file(
                'propeller/form_errors.html',
                context={
                    'errors': formset_errors,
                    'form': self.formset,
                    'layout': self.layout,
                }
            )
        return ''

    def _render(self):
        return '{}{}{}'.format(
            self.render_errors(),
            self.render_management_form(),
            self.render_forms(),
        )


class FormRenderer(BaseRenderer):
    """
    Default form renderer
    """

    def __init__(self, form, *args, **kwargs):
        if not isinstance(form, BaseForm):
            raise PropellerError(
                'Parameter "form" should contain a valid Django Form.')
        self.form = form
        super(FormRenderer, self).__init__(*args, **kwargs)

        # Handle form.empty_permitted
        if PROPELLER_SET_REQUIRED_SET_DISABLED and self.form.empty_permitted:
            self.set_required = False

        self.error_css_class = kwargs.get('error_css_class', None)
        self.required_css_class = kwargs.get('required_css_class', None)
        self.bound_css_class = kwargs.get('bound_css_class', None)

    def render_fields(self):
        rendered_fields = []
        for field in self.form:
            rendered_fields.append(render_field(
                field,
                layout=self.layout,
                form_group_class=self.form_group_class,
                field_class=self.field_class,
                label_class=self.label_class,
                show_label=self.show_label,
                show_help=self.show_help,
                exclude=self.exclude,
                set_required=self.set_required,  # Only relevant if DBS3_SET_REQUIRED_SET_DISABLED
                set_disabled=self.set_disabled,  # Only relevant if DBS3_SET_REQUIRED_SET_DISABLED
                set_placeholder=self.set_placeholder,
                size=self.size,
                horizontal_label_class=self.horizontal_label_class,
                horizontal_field_class=self.horizontal_field_class,
                error_css_class=self.error_css_class,
                required_css_class=self.required_css_class,
                bound_css_class=self.bound_css_class,
            ))
        return '\n'.join(rendered_fields)

    def get_fields_errors(self):
        form_errors = []
        for field in self.form:
            if not field.is_hidden and field.errors:
                form_errors += field.errors
        return form_errors

    def render_errors(self, _type='all'):
        form_errors = None
        if _type == 'all':
            form_errors = self.get_fields_errors() + self.form.non_field_errors()
        elif _type == 'fields':
            form_errors = self.get_fields_errors()
        elif _type == 'non_fields':
            form_errors = self.form.non_field_errors()

        if form_errors:
            return render_template_file(
                'propeller/form_errors.html',
                context={
                    'errors': form_errors,
                    'form': self.form,
                    'layout': self.layout,
                    'type': _type,
                }
            )

        return ''

    def _render(self):
        return self.render_errors() + self.render_fields()


class FieldRenderer(BaseRenderer):
    """
    Default field renderer
    """

    # These widgets will not be wrapped in a form-control class
    WIDGETS_NO_FORM_CONTROL = (
        CheckboxInput,
        RadioSelect,
        CheckboxSelectMultiple,
        FileInput,
    )

    def __init__(self, field, *args, **kwargs):
        if not isinstance(field, BoundField):
            raise PropellerError('Parameter "field" should contain a valid Django BoundField.')
        self.field = field
        super(FieldRenderer, self).__init__(*args, **kwargs)

        self.widget = field.field.widget
        self.is_multi_widget = isinstance(field.field.widget, MultiWidget)
        self.initial_attrs = self.widget.attrs.copy()
        self.field_help = text_value(mark_safe(field.help_text)) if self.show_help and field.help_text else ''
        self.field_errors = [conditional_escape(text_value(error)) for error in field.errors]

        if 'placeholder' in kwargs:
            # Find the placeholder in kwargs, even if it's empty
            self.placeholder = kwargs['placeholder']
        elif get_propeller_setting('set_placeholder'):
            # If not found, see if we set the label
            self.placeholder = field.label
        else:
            # Or just set it to empty
            self.placeholder = ''

        self.addon_before = kwargs.get('addon_before', self.widget.attrs.pop('addon_before', ''))
        self.addon_after = kwargs.get('addon_after', self.widget.attrs.pop('addon_after', ''))
        self.addon_before_class = kwargs.get('addon_before_class',
                                             self.widget.attrs.pop('addon_before_class', 'input-group-addon'))
        self.addon_after_class = kwargs.get('addon_after_class',
                                            self.widget.attrs.pop('addon_after_class', 'input-group-addon'))

        # These are set in Django or in the global PROPELLER settings, and
        # they can be overwritten in the template
        error_css_class = kwargs.get('error_css_class', None)
        required_css_class = kwargs.get('required_css_class', None)
        bound_css_class = kwargs.get('bound_css_class', None)
        if error_css_class is not None:
            self.error_css_class = error_css_class
        else:
            self.error_css_class = getattr(
                field.form, 'error_css_class',
                get_propeller_setting('error_css_class')
            )
        if required_css_class is not None:
            self.required_css_class = required_css_class
        else:
            self.required_css_class = getattr(
                field.form, 'required_css_class',
                get_propeller_setting('required_css_class')
            )
        if bound_css_class is not None:
            self.success_css_class = bound_css_class
        else:
            self.success_css_class = getattr(
                field.form, 'bound_css_class',
                get_propeller_setting('success_css_class')
            )

        # If the form is marked as form.empty_permitted, do not set required class
        if self.field.form.empty_permitted:
            self.required_css_class = ''

        # Special case to support Django 1.8 required / disabled
        if PROPELLER_SET_REQUIRED_SET_DISABLED:
            if self.field.form.empty_permitted:
                self.set_required = False
            self.set_disabled = kwargs.get('set_disabled', False)

    def restore_widget_attrs(self):
        self.widget.attrs = self.initial_attrs.copy()

    def add_class_attrs(self, widget=None):
        if widget is None:
            widget = self.widget
        classes = widget.attrs.get('class', '')
        if ReadOnlyPasswordHashWidget is not None and isinstance(widget, ReadOnlyPasswordHashWidget):
            # Render this is a static control
            classes = add_css_class(classes, 'form-control-static', prepend=True)
        elif not isinstance(widget, self.WIDGETS_NO_FORM_CONTROL):
            classes = add_css_class(classes, 'form-control', prepend=True)
            # For these widget types, add the size class here
            classes = add_css_class(classes, self.get_size_class())
        widget.attrs['class'] = classes

    def add_placeholder_attrs(self, widget=None):
        if widget is None:
            widget = self.widget
        placeholder = widget.attrs.get('placeholder', self.placeholder)
        if placeholder and self.set_placeholder and is_widget_with_placeholder(widget):
            # TODO: Should this be stripped and/or escaped?
            widget.attrs['placeholder'] = placeholder

    def add_help_attrs(self, widget=None):
        if widget is None:
            widget = self.widget
        if not isinstance(widget, CheckboxInput):
            widget.attrs['title'] = widget.attrs.get(
                'title',
                escape(strip_tags(self.field_help))
            )

    def add_required_attrs(self, widget=None):
        """
        Only relevant if DBS3_SET_REQUIRED_SET_DISABLED
        """
        if widget is None:
            widget = self.widget
        if self.set_required and is_widget_required_attribute(widget):
            widget.attrs['required'] = 'required'

    def add_disabled_attrs(self, widget=None):
        """
        Only relevant if DBS3_SET_REQUIRED_SET_DISABLED
        """
        if widget is None:
            widget = self.widget
        if self.set_disabled:
            widget.attrs['disabled'] = 'disabled'

    def add_widget_attrs(self):
        if self.is_multi_widget:
            widgets = self.widget.widgets
        else:
            widgets = [self.widget]
        for widget in widgets:
            self.add_class_attrs(widget)
            self.add_placeholder_attrs(widget)
            self.add_help_attrs(widget)
            if PROPELLER_SET_REQUIRED_SET_DISABLED:
                self.add_required_attrs(widget)
                self.add_disabled_attrs(widget)

    def list_to_class(self, html, klass):
        classes = add_css_class(klass, self.get_size_class())
        mapping = [
            ('<ul', '<div'),
            ('</ul>', '</div>'),
            ('<li', '<div class="{klass}"'.format(klass=classes)),
            ('</li>', '</div>'),
        ]
        for k, v in mapping:
            html = html.replace(k, v)
        return html

    def put_inside_label(self, html):
        content = '{field} {label}'.format(
            field=html,
            label=self.field.label,
        )
        return render_label(
            content=mark_safe(content),
            label_for=self.field.id_for_label,
            label_title=escape(strip_tags(self.field_help))
        )

    @staticmethod
    def fix_date_select_input(html):
        div1 = '<div class="col-xs-4">'
        div2 = '</div>'
        html = html.replace('<select', div1 + '<select')
        html = html.replace('</select>', '</select>' + div2)
        return '<div class="row bootstrap3-multi-input">' + html + '</div>'

    @staticmethod
    def fix_clearable_file_input(html):
        """
        Fix a clearable file input
        TODO: This needs improvement

        Currently Django returns
        Currently:
        <a href="dummy.txt">dummy.txt</a>
        <input id="file4-clear_id" name="file4-clear" type="checkbox" />
        <label for="file4-clear_id">Clear</label><br />
        Change: <input id="id_file4" name="file4" type="file" />
        <span class=help-block></span>
        </div>

        """
        # TODO This needs improvement
        return '<div class="row bootstrap3-multi-input"><div class="col-xs-12">{html}</div></div>'.format(
            html=html
        )

    def post_widget_render(self, html):
        if isinstance(self.widget, RadioSelect):
            html = self.list_to_class(html, 'radio')
        elif isinstance(self.widget, CheckboxSelectMultiple):
            html = self.list_to_class(html, 'checkbox')
        elif isinstance(self.widget, SelectDateWidget):
            html = self.fix_date_select_input(html)
        elif isinstance(self.widget, ClearableFileInput):
            html = self.fix_clearable_file_input(html)
        elif isinstance(self.widget, CheckboxInput):
            html = self.put_inside_label(html)
        return html

    def wrap_widget(self, html):
        if isinstance(self.widget, CheckboxInput):
            # Wrap checkboxes
            # Note checkboxes do not get size classes, see #318
            html = '<div class="checkbox">{content}</div>'.format(content=html)
        return html

    def make_input_group(self, html):
        if (self.addon_before or self.addon_after) and isinstance(self.widget, (TextInput, DateInput, Select)):
            before = '<span class="{input_class}">{addon}</span>'.format(
                input_class=self.addon_before_class, addon=self.addon_before) if self.addon_before else ''
            after = '<span class="{input_class}">{addon}</span>'.format(
                input_class=self.addon_after_class, addon=self.addon_after) if self.addon_after else ''
            html = '<div class="input-group">{before}{html}{after}</div>'.format(
                before=before,
                after=after,
                html=html
            )
        return html

    def append_to_field(self, html):
        help_text_and_errors = []
        if self.field_help:
            help_text_and_errors.append(self.field_help)
        help_text_and_errors += self.field_errors
        if help_text_and_errors:
            help_html = render_template_file(
                'propeller/field_help_text_and_errors.html',
                context={
                    'field': self.field,
                    'help_text_and_errors': help_text_and_errors,
                    'layout': self.layout,
                    'show_help': self.show_help,
                }
            )
            html += '<span class="help-block">{help}</span>'.format(help=help_html)
        return html

    def get_field_class(self):
        field_class = self.field_class
        if not field_class and self.layout == 'horizontal':
            field_class = self.horizontal_field_class
        return field_class

    def wrap_field(self, html):
        field_class = self.get_field_class()
        if field_class:
            html = '<div class="{klass}">{html}</div>'.format(
                klass=field_class, html=html)
        return html

    def get_label_class(self):
        label_class = self.label_class
        if not label_class and self.layout == 'horizontal':
            label_class = self.horizontal_label_class
        label_class = text_value(label_class)
        if not self.show_label:
            label_class = add_css_class(label_class, 'sr-only')
        return add_css_class(label_class, 'control-label')

    def get_label(self):
        if isinstance(self.widget, CheckboxInput):
            label = None
        else:
            label = self.field.label
        if self.layout == 'horizontal' and not label:
            return mark_safe('&#160;')
        return label

    def add_label(self, html):
        label = self.get_label()
        if label:
            html = render_label(
                label,
                label_for=self.field.id_for_label,
                label_class=self.get_label_class()) + html
        return html

    def get_form_group_class(self):
        form_group_class = self.form_group_class
        if self.field.errors:
            if self.error_css_class:
                form_group_class = add_css_class(form_group_class, self.error_css_class)
        else:
            if self.field.form.is_bound:
                form_group_class = add_css_class(form_group_class, self.success_css_class)
        if self.field.field.required and self.required_css_class:
            form_group_class = add_css_class(form_group_class, self.required_css_class)
        if self.layout == 'horizontal':
            form_group_class = add_css_class(
                form_group_class,
                self.get_size_class(prefix='form-group')
            )
        return form_group_class

    def wrap_label_and_field(self, html):
        return render_form_group(html, self.get_form_group_class())

    def _render(self):
        # See if we're not excluded
        if self.field.name in self.exclude.replace(' ', '').split(','):
            return ''
        # Hidden input requires no special treatment
        if self.field.is_hidden:
            return text_value(self.field)
        # Render the widget
        self.add_widget_attrs()
        html = self.field.as_widget(attrs=self.widget.attrs)
        self.restore_widget_attrs()
        # Start post render
        html = self.post_widget_render(html)
        html = self.wrap_widget(html)
        html = self.make_input_group(html)
        html = self.append_to_field(html)
        html = self.wrap_field(html)
        html = self.add_label(html)
        html = self.wrap_label_and_field(html)
        return html


class InlineFieldRenderer(FieldRenderer):
    """
    Inline field renderer
    """

    def add_error_attrs(self):
        field_title = self.widget.attrs.get('title', '')
        field_title += ' ' + ' '.join(
            [strip_tags(e) for e in self.field_errors])
        self.widget.attrs['title'] = field_title.strip()

    def add_widget_attrs(self):
        super(InlineFieldRenderer, self).add_widget_attrs()
        self.add_error_attrs()

    def append_to_field(self, html):
        return html

    def get_field_class(self):
        return self.field_class

    def get_label_class(self):
        return add_css_class(self.label_class, 'sr-only')



# -*- coding: utf-8 -*-

"""This module contains classes for constructing propeller navbars"""

from django.utils.safestring import mark_safe

from .utils import render_tag, add_css_class
from .text import text_concat

try:  # pragma: no cover
    from django.urls import reverse
except ImportError:  # pragma: no cover
    from django.core.urlresolvers import reverse


class NavBarLinkItem(object):

    """
    Generates a Link navbar item or a Link DropDown item.

    **Parameters**:

        name
            The display name for the item. (for example: 'Home')

        url
            The address for the link item. Can be a absolute URL or a resolvable Django url.
            (for example: 'http://example.org' or 'home'). Optional.

        icon
            not yet supported
    """

    name = None
    url = None
    icon = None

    def __init__(self, name="", url=None, icon=None):
        """

        """
        self.name = name
        self.url = url
        self.icon = icon

    def get_url(self):
        """
        Returns the url set in the attribute.

        **Returns**

            ``javascript:void(0);`` if ``url = None``

            or

            an absolute URL if ``url`` starts with 'http'

            or

            an relative URL if ``url`` is a resolvable Django url
        """
        if self.url:
            if not str(self.url).startswith('http'):
                return reverse(self.url)
            return self.url
        return "javascript:void(0);"

    def as_html(self):
        """Returns navbar link item as html"""
        tag = 'a'
        attrs = {'class': 'pmd-ripple-effect', 'href': self.get_url()}
        content = self.name
        return '<li>' + render_tag(tag, attrs=attrs, content=mark_safe(content), ) + '</li>'


class NavBarDropDownDivider(object):

    """Generates a DropDown Divider item."""

    @staticmethod
    def as_html():
        """Returns navbar dropdown divider as html"""
        tag = 'li'
        attrs = {'role': 'separator', 'class': 'divider'}
        return render_tag(tag, attrs=attrs, )


class NavBarDropDownItem(NavBarLinkItem):

    """
    Generates a DropDown navbar item.

    **Parameters**:

        name
            The display name for the item. (for example: 'Home')

        url
            The address for the link item. Can be a absolute URL or a resolvable Django url.
            (for example: 'http://example.org' or 'home'). Optional.

        icon
            not yet supported

        items
            A list containing NavBarLinkItems and/or NavBarDropDownDivider. Optional.
    """

    items = []

    def __init__(self, name="", items=None, url=None):
        super(NavBarDropDownItem, self).__init__(name, url)
        if items:
            self.items = items

    def as_html(self):
        """Returns navbar dropdown item as html"""
        tag = 'li'
        attrs = {'class': 'dropdown pmd-dropdown'}
        content = '<a data-toggle="dropdown" class="pmd-ripple-effect dropdown-toggle" data-sidebar="true" ' \
                  'href="%s">%s<span class="caret"></span></a>' % (self.url, self.name)
        content = text_concat(content, '<ul class="dropdown-menu">')
        for itm in self.items:
            content = text_concat(content, itm.as_html())
        content = text_concat(content, '</ul>')

        return render_tag(tag, attrs=attrs, content=mark_safe(content), )


class NavBar(object):

    """
    NavBar is a class that generates a NavBar.

    **Parameters**:

        brandname
            The brand shown on the very left of the navbar.

        brandurl
            The address for the brand name. Can be a absolute URL or a resolvable Django url.
            (for example: 'http://example.org' or 'home'). Optional.

        items
            A list containing NavBarLinkItems and/or NavBarDropDownItems. Optional.

        style_inverse
            Generate a dark navbar if true (default) or a light navbar if false.

        style_static
            Sets the static style for the navbar. Static if true (default) or floating on top if false.
    """

    brandname = ""
    brandurl = None
    items = []
    style_inverse = True
    style_static = True

    def get_brand_url(self):
        """
        Returns the brand url set in the attribute.

        **Returns**

            ``javascript:void(0);`` if ``brandurl = None``

            or

            an absolute URL if ``brandurl`` starts with 'http'

            or

            an relative URL if ``brandurl`` is a resolvable Django url
        """
        if self.brandurl:
            if not str(self.brandurl).startswith('http'):
                return reverse(self.brandurl)
            return self.brandurl
        return "javascript:void(0);"

    @staticmethod
    def render_toggle():
        """Returns navbar toggle as html (for responsive)"""
        tag = 'button'
        attrs = {
            'class': 'navbar-toggle collapsed',
            'type': 'button',
            'data-toggle': 'collapse',
            'aria-expanded': 'false'
        }
        content = '<span class="sr-only">Toggle navigation</span>'
        content = text_concat(content, '<span class="icon-bar"></span>')
        content = text_concat(content, '<span class="icon-bar"></span>')
        content = text_concat(content, '<span class="icon-bar"></span>')
        return render_tag(tag, attrs=attrs, content=mark_safe(content), )

    def render_header(self):
        """Returns navbar header as html"""
        tag = 'div'
        attrs = {'class': 'navbar-header'}
        content = self.render_toggle()
        content = text_concat(content, '<a href="%s" class="navbar-brand navbar-brand-custome">%s'
                                       '</a>' % (self.get_brand_url(), self.brandname))
        return render_tag(tag, attrs=attrs, content=mark_safe(content), )

    def render_items(self):
        """Returns navbar items as html (for item container)"""
        tag = 'ul'
        attrs = {'class': 'nav navbar-nav'}
        content = ''
        for itm in self.items:
            content = text_concat(content, itm.as_html())
        return render_tag(tag, attrs=attrs, content=mark_safe(content), )

    def render_item_container(self):
        """Returns navbar items as html"""
        tag = 'div'
        attrs = {'class': 'collapse navbar-collapse'}
        content = self.render_items()
        return render_tag(tag, attrs=attrs, content=mark_safe(content), )

    def render_content(self):
        """Returns navbar content as html"""
        tag = 'div'
        attrs = {'class': 'container-fluid'}
        content = self.render_header()
        content = text_concat(content, self.render_item_container())
        return render_tag(tag, attrs=attrs, content=mark_safe(content), )

    def as_html(self):
        """Returns navbar as html"""
        tag = 'nav'
        classes = 'navbar'
        if self.style_inverse:
            classes = add_css_class(classes, 'navbar-inverse')
        if self.style_static:
            classes = add_css_class(classes, 'navbar-static')
        else:
            classes = add_css_class(classes, 'navbar-top')
        classes = add_css_class(classes, 'pmd-navbar')
        classes = add_css_class(classes, 'pmd-z-depth')
        attrs = {'class': classes}
        content = self.render_content()
        content = text_concat(content, '<div class="pmd-sidebar-overlay"></div>')
        return render_tag(tag, attrs=attrs, content=mark_safe(content), )


class CustomItem(object):
    """
    Returns a custom NavBar item.

    Just assign some raw HTML code to `html` attribute.
    """
    html = ""

    def __init__(self, html=""):
        self.html = html
    
    def as_html(self):
        return self.html



# -*- coding: utf-8 -*-

"""Provides expected test results"""

RESULT_CARD1 = '<div class="pmd-card pmd-card-default pmd-z-depth col-md-4">' \
               '<div class="pmd-card-title">' \
               '<div class="media-left">' \
               '<img src="http://propeller.in/assets/images/avatar-icon-40x40.png" width="40" height="40">' \
               '</div>' \
               '<div class="media-body media-middle">' \
               '<h3 class="pmd-card-title-text">Two-line item</h3>' \
               '<span class="pmd-card-subtitle-text">Secondary text</span>' \
               '</div>' \
               '</div>' \
               '<img src="http://propeller.in/assets/images/profile-pic.png" class="img-responsive">' \
               '<div class="pmd-card-title">' \
               '<h2 class="pmd-card-title-text">Title goes here</h2>' \
               '<span class="pmd-card-subtitle-text">Secondary text</span>' \
               '</div>' \
               '<div class="pmd-card-body">' \
               'Cards provide context and an entry point to more robust information and views. ' \
               'Don\'t overload cards with extraneous information or actions.' \
               '</div>' \
               '<div class="pmd-card-actions">' \
               '<button class="btn btn-sm pmd-btn-fab pmd-btn-flat pmd-ripple-effect btn-primary" type="button">' \
               '<i class="material-icons pmd-sm">share</i>' \
               '</button>' \
               '<button class="btn btn-sm pmd-btn-fab pmd-btn-flat pmd-ripple-effect btn-primary" type="button">' \
               '<i class="material-icons pmd-sm">thumb_up</i>' \
               '</button>' \
               '<button class="btn btn-sm pmd-btn-fab pmd-btn-flat pmd-ripple-effect btn-primary" type="button">' \
               '<i class="material-icons pmd-sm">drafts</i>' \
               '</button>' \
               '</div>' \
               '<div class="pmd-card-actions">' \
               '<button class="btn btn-primary pmd-ripple-effect pmd-btn-default" href="#" type="button">primary' \
               '</button>' \
               '<button class="btn btn-default pmd-ripple-effect pmd-btn-default" href="#" type="button">Action' \
               '</button>' \
               '<button class="btn btn-default pmd-ripple-effect pmd-btn-default" href="#" type="button">third' \
               '</button>' \
               '</div>' \
               '</div>'

RESULT_CARD2 = '<div class="pmd-card pmd-card-default pmd-z-depth col-md-4">' \
               '<img src="http://propeller.in/assets/images/profile-pic.png" class="img-responsive">' \
               '<div class="pmd-card-title">' \
               '<h2 class="pmd-card-title-text">Title goes here</h2>' \
               '<span class="pmd-card-subtitle-text">Secondary text</span>' \
               '</div>' \
               '<div class="pmd-card-actions">' \
               '<button class="btn btn-sm pmd-btn-fab pmd-btn-flat pmd-ripple-effect btn-primary" type="button">' \
               '<i class="material-icons pmd-sm">share</i>' \
               '</button>' \
               '<button class="btn btn-sm pmd-btn-fab pmd-btn-flat pmd-ripple-effect btn-primary" type="button">' \
               '<i class="material-icons pmd-sm">thumb_up</i>' \
               '</button>' \
               '<button class="btn btn-sm pmd-btn-fab pmd-btn-flat pmd-ripple-effect btn-primary" type="button">' \
               '<i class="material-icons pmd-sm">drafts</i>' \
               '</button>' \
               '</div>' \
               '</div>'

RESULT_CARD3 = '<div class="pmd-card pmd-card-inverse pmd-z-depth col-md-4">' \
               '<div class="pmd-card-title">' \
               '<div class="media-left">' \
               '<img src="http://propeller.in/assets/images/avatar-icon-40x40.png" width="40" height="40">' \
               '</div>' \
               '<div class="media-body media-middle">' \
               '<h3 class="pmd-card-title-text">Two-line item</h3>' \
               '<span class="pmd-card-subtitle-text">Secondary text</span>' \
               '</div>' \
               '</div>' \
               '<img src="http://propeller.in/assets/images/profile-pic.png" class="img-responsive">' \
               '<div class="pmd-card-title">' \
               '<h2 class="pmd-card-title-text">Title goes here</h2>' \
               '<span class="pmd-card-subtitle-text">Secondary text</span>' \
               '</div>' \
               '<div class="pmd-card-body">' \
               'Cards provide context and an entry point to more robust information and views. ' \
               'Don\'t overload cards with extraneous information or actions.' \
               '</div>' \
               '<div class="pmd-card-actions">' \
               '<button class="btn btn-sm pmd-btn-fab pmd-btn-flat pmd-ripple-effect btn-primary" type="button">' \
               '<i class="material-icons pmd-sm">share</i>' \
               '</button>' \
               '<button class="btn btn-sm pmd-btn-fab pmd-btn-flat pmd-ripple-effect btn-primary" type="button">' \
               '<i class="material-icons pmd-sm">thumb_up</i>' \
               '</button>' \
               '<button class="btn btn-sm pmd-btn-fab pmd-btn-flat pmd-ripple-effect btn-primary" type="button">' \
               '<i class="material-icons pmd-sm">drafts</i>' \
               '</button>' \
               '</div>' \
               '<div class="pmd-card-actions">' \
               '<button class="btn btn-primary pmd-ripple-effect pmd-btn-default" href="#" type="button">primary' \
               '</button>' \
               '<button class="btn btn-default pmd-ripple-effect pmd-btn-default" href="#" type="button">Action' \
               '</button>' \
               '<button class="btn btn-default pmd-ripple-effect pmd-btn-default" href="#" type="button">third' \
               '</button>' \
               '</div>' \
               '</div>'

RESULT_CARD4 = '<div class="pmd-card pmd-card-media-inline pmd-card-default pmd-z-depth col-md-4">' \
               '<img src="http://propeller.in/assets/images/profile-pic.png" width="80" height="80">' \
               '<div class="pmd-card-actions">' \
               '<button class="btn btn-primary pmd-ripple-effect pmd-btn-default" href="#" type="button">primary' \
               '</button>' \
               '<button class="btn btn-default pmd-ripple-effect pmd-btn-default" href="#" type="button">Action' \
               '</button>' \
               '<button class="btn btn-default pmd-ripple-effect pmd-btn-default" href="#" type="button">third' \
               '</button>' \
               '</div>' \
               '</div>'

RESULT_CARD5 = '<div class="pmd-card pmd-card-media-inline pmd-card-default pmd-z-depth col-md-4">' \
               '<div class="pmd-card-media">' \
               '<div class="media-body"></div>' \
               '<div class="media-right media-middle">' \
               '<img src="http://propeller.in/assets/images/profile-pic.png" width="80" height="80">' \
               '</div>' \
               '</div>' \
               '<div class="pmd-card-actions">' \
               '<button class="btn btn-primary pmd-ripple-effect pmd-btn-default" href="#" type="button">primary' \
               '</button>' \
               '<button class="btn btn-default pmd-ripple-effect pmd-btn-default" href="#" type="button">Action' \
               '</button>' \
               '<button class="btn btn-default pmd-ripple-effect pmd-btn-default" href="#" type="button">third' \
               '</button>' \
               '</div>' \
               '</div>'

RESULT_NAVBAR1 = '<nav class="navbar navbar-inverse navbar-static pmd-navbar pmd-z-depth">' \
                 '<div class="container-fluid">' \
                 '<div class="navbar-header">' \
                 '<button aria-expanded="false" class="navbar-toggle collapsed" data-toggle="collapse" type="button">' \
                 '<span class="sr-only">Toggle navigation</span>' \
                 '<span class="icon-bar">' \
                 '</span><span class="icon-bar">' \
                 '</span><span class="icon-bar">' \
                 '</span>' \
                 '</button>' \
                 '<a href="https://github.com/tfroehlich82/django-propeller" ' \
                 'class="navbar-brand navbar-brand-custome">propeller-test</a>' \
                 '</div>' \
                 '<div class="collapse navbar-collapse">' \
                 '<ul class="nav navbar-nav">' \
                 '<li><a class="pmd-ripple-effect" href="javascript:void(0);">Test1</a></li>' \
                 '<li class="dropdown pmd-dropdown"><a data-toggle="dropdown" ' \
                 'class="pmd-ripple-effect dropdown-toggle" data-sidebar="true" href="None">Test2' \
                 '<span class="caret"></span></a>' \
                 '<ul class="dropdown-menu">' \
                 '<li><a class="pmd-ripple-effect" href="javascript:void(0);">Test3</a></li>' \
                 '<li><a class="pmd-ripple-effect" href="javascript:void(0);">Test4</a></li>' \
                 '<li class="divider" role="separator"></li>' \
                 '<li><a class="pmd-ripple-effect" href="javascript:void(0);">Test5</a></li>' \
                 '</ul>' \
                 '</li>' \
                 '</ul>' \
                 '</div>' \
                 '</div>' \
                 '<div class="pmd-sidebar-overlay"></div>' \
                 '</nav>'

RESULT_NAVBAR2 = '<nav class="navbar navbar-inverse navbar-top pmd-navbar pmd-z-depth">' \
                 '<div class="container-fluid">' \
                 '<div class="navbar-header">' \
                 '<button aria-expanded="false" class="navbar-toggle collapsed" data-toggle="collapse" type="button">' \
                 '<span class="sr-only">Toggle navigation</span>' \
                 '<span class="icon-bar"></span>' \
                 '<span class="icon-bar"></span>' \
                 '<span class="icon-bar"></span>' \
                 '</button>' \
                 '<a href="javascript:void(0);" class="navbar-brand navbar-brand-custome">propeller-test</a>' \
                 '</div>' \
                 '<div class="collapse navbar-collapse">' \
                 '<ul class="nav navbar-nav">' \
                 '<li><a class="pmd-ripple-effect" href="javascript:void(0);">Test1</a></li>' \
                 '<li class="dropdown pmd-dropdown"><a data-toggle="dropdown" ' \
                 'class="pmd-ripple-effect dropdown-toggle" data-sidebar="true" href="None">Test2' \
                 '<span class="caret"></span></a>' \
                 '<ul class="dropdown-menu">' \
                 '<li><a class="pmd-ripple-effect" href="javascript:void(0);">Test3</a></li>' \
                 '<li><a class="pmd-ripple-effect" href="javascript:void(0);">Test4</a></li>' \
                 '<li class="divider" role="separator"></li>' \
                 '<li><a class="pmd-ripple-effect" href="javascript:void(0);">Test5</a></li>' \
                 '</ul>' \
                 '</li>' \
                 '</ul>' \
                 '</div>' \
                 '</div>' \
                 '<div class="pmd-sidebar-overlay"></div>' \
                 '</nav>'

RESULT_NAVBAR3 = '<nav class="navbar navbar-top pmd-navbar pmd-z-depth">' \
                 '<div class="container-fluid">' \
                 '<div class="navbar-header">' \
                 '<button aria-expanded="false" class="navbar-toggle collapsed" data-toggle="collapse" type="button">' \
                 '<span class="sr-only">Toggle navigation</span>' \
                 '<span class="icon-bar"></span>' \
                 '<span class="icon-bar"></span>' \
                 '<span class="icon-bar"></span>' \
                 '</button>' \
                 '<a href="https://github.com/tfroehlich82/django-propeller" ' \
                 'class="navbar-brand navbar-brand-custome">propeller-test</a>' \
                 '</div>' \
                 '<div class="collapse navbar-collapse">' \
                 '<ul class="nav navbar-nav">' \
                 '<li><a class="pmd-ripple-effect" href="javascript:void(0);">Test1</a></li>' \
                 '<li class="dropdown pmd-dropdown">' \
                 '<a data-toggle="dropdown" class="pmd-ripple-effect dropdown-toggle" ' \
                 'data-sidebar="true" href="None">Test2<span class="caret"></span></a>' \
                 '<ul class="dropdown-menu">' \
                 '<li><a class="pmd-ripple-effect" href="javascript:void(0);">Test3</a></li>' \
                 '<li><a class="pmd-ripple-effect" href="javascript:void(0);">Test4</a></li>' \
                 '<li class="divider" role="separator"></li>' \
                 '<li><a class="pmd-ripple-effect" href="javascript:void(0);">Test5</a></li>' \
                 '</ul>' \
                 '</li>' \
                 '</ul>' \
                 '</div>' \
                 '</div>' \
                 '<div class="pmd-sidebar-overlay"></div>' \
                 '</nav>'



# -*- coding: utf-8 -*-
from django.views.generic.base import ContextMixin


class NavBarMixin(ContextMixin):
    navbar_class = None
    navbar_name = 'navbar'

    def get_context_data(self, **kwargs):
        context = super(NavBarMixin, self).get_context_data(**kwargs)
        context[self.navbar_name] = self.navbar_class
        return context



# -*- coding: utf-8 -*-
from __future__ import unicode_literals


try:  # pragma: no cover
    from django.utils.encoding import force_text
except ImportError:  # pragma: no cover
    from django.utils.encoding import force_unicode as force_text


def text_value(value):
    """
    Force a value to text, render None as an empty string
    """
    if value is None:
        return ''
    return force_text(value)


def text_concat(*args, **kwargs):
    """
    Concatenate several values as a text string with an optional separator
    """
    separator = text_value(kwargs.get('separator', ''))
    values = filter(None, [text_value(v) for v in args])
    return separator.join(values)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import sys

import django2_propeller

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

version = django2_propeller.__version__

if sys.argv[-1] == 'publish':
    os.system('cd docs && make html')
    os.system('python -m twine upload --repository-url https://test.pypi.org/legacy/ dist/*')
    sys.exit()

if sys.argv[-1] == 'test':
    print("Running tests only on current environment.")
    print("Use `tox` for testing multiple environments.")
    os.system('python manage.py test')
    sys.exit()

with open('README.md') as readme_file:
    readme = readme_file.read()

setup(
    name='django2-propeller',
    version=version,
    description="""Propeller support for Django projects""",
    long_description=readme + '\n',
    author='Adam Radestock',
    author_email='raddishiow@gmail.com   ',
    url='https://github.com/RaddishIoW/django2-propeller',
    packages=[
        'django2_propeller',
    ],
    include_package_data=True,
    install_requires=[
        'Django>=2.0,<3.0'
    ],
    license="MIT License",
    zip_safe=False,
    keywords='django2-propeller',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Operating System :: OS Independent',
        'Topic :: Software Development :: Libraries',
        'Topic :: Utilities',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'Framework :: Django :: 2.0',
        'Framework :: Django :: 2.1',
        'Framework :: Django :: 2.2',
    ],
)
